
import logging
log = logging.getLogger("Thug")


class SchemeHandler(object):
    def __init__(self):
        pass

    def handle_hcp(self, window, url):
        log.warning('Microsoft Internet Explorer HCP Scheme Detected')

        hcp = url.split('svr=')
        if len(hcp) < 2:
            return

        hcp = hcp[1].split('defer>')
        if len(hcp) < 2:
            return

        hcp = hcp[1].split('</script')
        if not hcp:
            return

        log.ThugLogging.add_behavior_warn('Microsoft Windows Help Center Malformed Escape Sequences Incorrect Handling',
                                          'CVE-2010-1885')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2010-1885", None)

        if not hcp[0]:
            return

        window.evalScript(hcp[0])

    def handle_res(self, window, url):
        log.warning('Microsoft Internet Explorer RES Scheme Detected')

        try:
            log.URLClassifier.classify(url)
        except:  # pylint:disable=bare-except
            pass



# OWC10/11.Spreadsheet ActiveX
# CVE-2009-1136

import logging

log = logging.getLogger("Thug")


def _Evaluate(self, *args):
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "OWC 10/11.Spreadsheet ActiveX",
                                      "Attack in _Evaluate function",
                                      cve = "CVE-2009-1136")

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2009-1136", None)


def Evaluate(self, *args):
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "OWC 10/11.Spreadsheet ActiveX",
                                      "Attack in Evaluate function",
                                      cve = "CVE-2009-1136")

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2009-1136", None)



#!/usr/bin/env python
#
# MAEC11.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import os
import logging
import datetime

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from . import MAEC_v1_1 as maec

NAMESPACEDEF_ = 'xmlns:ns1="http://xml/metadataSharing.xsd" xmlns="http://maec.mitre.org/XMLSchema/maec-core-1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://maec.mitre.org/XMLSchema/maec-core-1 file:MAEC_v1.1.xsd"'

log = logging.getLogger("Thug")


class MAEC11(object):
    def __init__(self, thug_version):
        self._tools = ({
                        'id'            : 'maec:thug:tol:1',
                        'Name'          : 'Thug',
                        'Version'       : thug_version,
                        'Vendor'        : None,
                        'Organization'  : 'The Honeynet Project',
                       }, )

        self.id              = self.__make_counter(2)
        self.associated_code = None
        self.object_pool     = None
        self.signatures      = list()
        self.cached_data     = None

        self.__init_tools_used()
        self.__create_maec_bundle()
        self.__add_analysis_to_analyses()
        self.__add_subject_to_analysis()

    def __make_counter(self, p):
        _id = p
        while True:
            yield _id
            _id += 1

    def __init_tools_used(self):
        self.tools_used = maec.Tools_Used()

        for t in self._tools:
            tool = maec.ToolType(id           = t['id'],
                                 Name         = t['Name'],
                                 Version      = t['Version'],
                                 Vendor       = t['Vendor'],
                                 Organization = t['Organization'])

            self.tools_used.add_Tool(tool)

    def __create_maec_bundle(self):
        self.analyses    = maec.Analyses()
        self.behaviors   = maec.Behaviors()
        self.pools       = maec.Pools()
        self.maec_bundle = maec.BundleType(id             = "maec:thug:bnd:1",
                                           Analyses       = self.analyses,
                                           Behaviors      = self.behaviors,
                                           Pools          = self.pools,
                                           schema_version = 1.1)

    def __create_analysis(self):
        return maec.AnalysisType(id              = 'maec:thug:ana:%d' % (next(self.id)),
                                 start_datetime  = datetime.datetime.now(),
                                 analysis_method = "Dynamic",
                                 Tools_Used      = self.tools_used)

    def __add_analysis_to_analyses(self):
        analyses      = self.maec_bundle.get_Analyses()
        self.analysis = self.__create_analysis()

        analyses.add_Analysis(self.analysis)

    def __add_subject_to_analysis(self):
        self.subject = maec.Subject()

    @property
    def maec11_enabled(self):
        return log.ThugOpts.maec11_logging or 'maec11' in log.ThugLogging.formats

    def finalize_analysis(self):
        self.analysis.set_complete_datetime(datetime.datetime.now())

    def create_object(self, url):
        object_id = "maec:thug:obj:%d" % (next(self.id))

        internet_object_attributes = maec.Internet_Object_Attributes(URI = url)

        return maec.ObjectType(type_                      = "URI",
                               object_name                = url,
                               Internet_Object_Attributes = internet_object_attributes,
                               id                         = object_id)

    def _add_object_to_subject(self, url):
        self.object = self.create_object(url)
        self.subject.set_Object(self.object)

    def set_url(self, url):
        if not self.maec11_enabled:
            return

        self._add_object_to_subject(url.decode('utf-8'))

    def _add_associated_code_to_object(self):
        if self.associated_code:
            return

        self.associated_code = maec.Associated_Code()
        self.object.set_Associated_Code(self.associated_code)

    def _normalize_snippet(self, snippet):
        _snippet = '\n'
        for line in snippet.splitlines():
            _snippet += 5 * '\t' + line + '\n'
        _snippet += 4 * '\t'

        try:
            return _snippet.encode('ascii', 'ignore')
        except: #pylint:disable=bare-except
            return _snippet.decode('ascii', 'ignore')

    def _add_snippet_to_associated_code(self, snippet, language, relationship, method = "Dynamic Analysis"):
        discovery_method = self._create_discovery_method(method)

        code = self._create_code_segment(self._normalize_snippet(snippet),
                                        language,
                                        discovery_method)

        snippet = maec.Associated_Code_Snippet()
        snippet.set_Code_Snippet(code)
        snippet.set_Nature_Of_Relationship(relationship)
        self.associated_code.add_Associated_Code_Snippet(snippet)

    def add_code_snippet(self, snippet, language, relationship, method = "Dynamic Analysis"):
        if not self.maec11_enabled:
            return

        self._add_associated_code_to_object()
        self._add_snippet_to_associated_code(snippet, language, relationship, method)

    def _create_code_segment(self, snippet, language, discovery_method):
        return maec.CodeType(Code_Segment     = snippet,
                             Discovery_Method = discovery_method,
                             language         = language,
                             xorpattern       = None,
                             id               = "maec:thug:cde:%d" % (next(self.id)))

    def _create_discovery_method(self, method, tool = "Thug"):
        _tool_id = None

        for p in self._tools:
            if p["Name"] == tool:
                _tool_id = p["id"]
                break

        return maec.DiscoveryMethod(method  = method,
                                    tool_id = _tool_id if _tool_id else "maec:thug:tol:%d" % (next(self.id)))

    def add_behavior(self, description = None, cve = None, method = "Dynamic Analysis"):
        if not self.maec11_enabled:
            return

        if not cve and not description:
            return

        _id      = "maec:thug:bhv:%s" % (next(self.id))
        behavior = maec.BehaviorType(id = _id)
        behavior.set_Discovery_Method(self._create_discovery_method(method))

        purpose = maec.Purpose()

        if cve:
            t = maec.Attempted_Vulnerability_Exploit()
            t.set_vulnerability_type('Known')

            if cve:
                c = maec.CVEVulnerabilityType(cve_id = cve)
                t.set_Known_Exploit(c)

            purpose.set_Attempted_Vulnerability_Exploit(t)
            behavior.set_Purpose(purpose)

        if description:
            desc = maec.StructuredTextType()

            try:
                desc.add_Text(description)
            except: #pylint:disable=bare-except
                desc.add_Text(description.decode('utf-8'))

            behavior.set_Description(desc)

        self.behaviors.add_Behavior(behavior)

    def add_behavior_warn(self, description = None, cve = None, method = "Dynamic Analysis"):
        if not self.maec11_enabled:
            return

        self.add_behavior(description, cve, method)

    def _check_signature(self, signature):
        if not signature:
            return True

        if not self.pools.get_Object_Pool() or not self.signatures:
            self.signatures.append(signature)
            return False

        for p in [s for s in self.signatures if s['type'] == signature['type']]:
            if p['md5'] == signature['md5'] and p['sha1'] == signature['sha1']:
                return True

        self.signatures.append(signature)
        return False

    def _add_object(self, signature):
        if self._check_signature(signature):
            return

        hashes    = maec.Hashes()
        file_type = None

        for item in signature:
            if item in ('url', 'data', ):
                continue

            if item in ('type', ):
                file_type = signature[item]
                continue

            _hash = maec.HashType(type_     = item,
                                 Hash_Value = signature[item])

            hashes.add_Hash(_hash)

        if not file_type:
            return

        _file_type = maec.File_Type(type_ = file_type)
        filesystem = maec.File_System_Object_Attributes(Hashes    = hashes,
                                                        File_Type = _file_type)

        _object = maec.ObjectType(id = "maec:thug:obj:%d" % (next(self.id)))
        _object.set_File_System_Object_Attributes(filesystem)

        if self.object_pool is None:
            self.object_pool = maec.Object_Pool()
            self.pools.set_Object_Pool(self.object_pool)

        self.object_pool.add_Object(_object)

    def log_file(self, data, url = None, params = None):
        if not self.maec11_enabled:
            return

        self._add_object(data)

    def export(self, basedir):
        if not self.maec11_enabled:
            return

        output = StringIO()
        self.maec_bundle.export(output,
                                0,
                                name_         = 'MAEC_Bundle',
                                namespace_    = '',
                                namespacedef_ = NAMESPACEDEF_)

        if log.ThugOpts.maec11_logging and log.ThugOpts.file_logging:
            logdir = os.path.join(basedir, "analysis", "maec11")
            log.ThugLogging.store_content(logdir, 'analysis.xml', output.getvalue())

        self.cached_data = output

    def get_maec11_data(self, basedir):
        if self.cached_data:
            return self.cached_data.getvalue()

        return None



#!/usr/bin/env python

import logging

log = logging.getLogger("Thug")


# Introduced in DOM Level 2
class Event(object):
    CAPTURING_PHASE     = 1  # The current event phase is the capturing phase.
    AT_TARGET           = 2  # The event is currently being evaluated at the target EventTarget
    BUBBLING_PHASE      = 3  # The current event phase is the bubbling phase.

    def __init__(self, target):
        self._target             = target
        self.currentTarget       = target
        self.eventPhase          = self.AT_TARGET
        self._stoppedPropagation = False
        self._defaultPrevented   = False
        self._canBubble          = False
        self._cancelable         = False

        # Prior to IE9, IE does not support the stopPropagation() method. Instead,
        # the IE Event object has a property named `cancelBubble'. Setting this
        # property to true prevents any further propagation (IE8 and before do not
        # support the captuting phase of event propagation so bubbling is the only
        # kind of propagation to be canceled)
        if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion < 9:
            self.cancelBubble = property(self._getPropagationStatus, self._setPropagationStatus)
        else:
            self.stopPropagation = self._stopPropagation

        # In IE prior to IE9 the default action can be canceled by setting the
        # `returnValue' of the Event object to false
        if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion < 9:
            self.returnValue = property(self._getDefaultPrevented, self._setDefaultPrevented)
        else:
            self.preventDefault = self._preventDefault

    def _getPropagationStatus(self):
        return self._stoppedPropagation

    def _setPropagationStatus(self, value):
        self._stoppedPropagation = True if value else False

    def _stopPropagation(self):
        self._stoppedPropagation = True

    def _getDefaultPrevented(self):
        return self._defaultPrevented

    def _setDefaultPrevented(self, value):
        self._defaultPrevented = True if value else False

    def _preventDefault(self):
        self._defaultPrevented = True

    @property
    def type(self):
        return self._type

    @property
    def target(self):
        return self._target

    @property
    def bubbles(self):
        return self._canBubble

    @property
    def cancelable(self):
        return self._cancelable

    @property
    def timeStamp(self):
        return 0

    def initEvent(self, eventTypeArg, canBubbleArg, cancelableArg):
        self._type       = eventTypeArg
        self._canBubble  = canBubbleArg
        self._cancelable = cancelableArg



# jetAudio "DownloadFromMusicStore()" Arbitrary File Download Vulnerability
# CVE-2007-4983

import logging

log = logging.getLogger("Thug")


def DownloadFromMusicStore(self, url, dst, title, artist, album, genere, size, param1, param2):
    log.ThugLogging.add_behavior_warn('[JetAudio ActiveX] Downloading from URL %s (saving locally as %s)' % (url, dst, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "JetAudio ActiveX",
                                      "Downloading from URL",
                                      cve = "CVE-2007-4983",
                                      data = {
                                                "url" : url,
                                                "file": dst
                                             },
                                      forward = False)

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4983", None)

    try:
        self._window._navigator.fetch(url, redirect_type = "JetAudio exploit")
    except:  # pylint:disable=bare-except
        log.ThugLogging.add_behavior_warn('[JetAudio ActiveX] Fetch failed')



# D-Link MPEG4 SHM Audio Control
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def SetUrl(self, val):
    self.__dict__['Url'] = val
    if len(val) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "D-Link MPEG4 SHM Audio Control ActiveX",
                                          "Overflow in Url property")



#!/usr/bin/env python
#
# Window.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import os
import sched
import time
import logging
import PyV8
# import traceback
import urllib
import bs4 as BeautifulSoup
import new
import numbers
import collections
import datetime
import six
from .W3C import w3c
from .W3C.HTML.HTMLCollection import HTMLCollection
from .JSClass import JSClass
from .Navigator import Navigator
from .Location import Location
from .Screen import Screen
from .History import History
from .ClipboardData import ClipboardData
from .External import External
from .Sidebar import Sidebar
from .Chrome import Chrome
from .Opera import Opera
from .Console import Console
from .Components import Components
from .Crypto import Crypto
from .CCInterpreter import CCInterpreter
from .LocalStorage import LocalStorage
from .SessionStorage import SessionStorage
from .Map import Map
from .MozConnection import mozConnection
from .w3c_bindings import w3c_bindings
from thug.ActiveX.ActiveX import _ActiveXObject
from thug.AST.AST import AST
from thug.Debugger import Shellcode
from thug.Java.java import java

sched = sched.scheduler(time.time, time.sleep)
log = logging.getLogger("Thug")

import thug


class Window(JSClass):

    class Timer(object):
        def __init__(self, window, code, delay, repeat, lang = 'JavaScript'):
            self.window  = window
            self.code    = code
            self.delay   = float(delay) / 1000
            self.repeat  = repeat
            self.lang    = lang
            self.running = True

        def start(self):
            self.event = sched.enter(self.delay, 1, self.execute, ())
            try:
                sched.run()
            except:  # pylint:disable=bare-except
                pass

        def stop(self):
            self.running = False
            if self.event in sched.queue:
                sched.cancel(self.event)

        def execute(self):
            if not self.running:
                return

            with self.window.context as ctx:
                if isinstance(self.code, six.string_types):
                    return ctx.eval(self.code)
                elif isinstance(self.code, PyV8.JSFunction):
                    return self.code()
                else:
                    log.warning("Error while handling Window timer")
                    return

            if self.repeat:
                self.start()

    def __init__(self, url, dom_or_doc, navigator = None, personality = 'winxpie60', name="",
                 target='_blank', parent = None, opener = None, replace = False, screen = None,
                 width = 800, height = 600, left = 0, top = 0, **kwds):

        self.url = url
        self.doc = w3c.getDOMImplementation(dom_or_doc, **kwds) if isinstance(dom_or_doc, BeautifulSoup.BeautifulSoup) else dom_or_doc

        self.doc.window        = self
        self.doc.contentWindow = self

        for p in w3c_bindings:
            setattr(self, p, w3c_bindings[p])

        self._navigator = navigator if navigator else Navigator(personality, self)
        self._location  = Location(self)
        self._history   = History(self)

        if url not in ('about:blank', ):
            self._history.update(url, replace)

        self.doc.location = property(self.getLocation, self.setLocation)

        self._target = target
        self._parent = parent if parent else self
        self._opener = opener
        self._screen = screen or Screen(width, height, 32)
        self._closed = False

        self._personality = personality
        self.__init_personality()

        self.name          = name
        self.defaultStatus = ""
        self.status        = ""
        self._left         = left
        self._top          = top
        self.innerWidth    = width
        self.innerHeight   = height
        self.outerWidth    = width
        self.outerHeight   = height
        self.timers        = []
        self.java          = java()

        self._symbols      = set()
        self._methods      = tuple()
        self._frames       = set()
        self._inner_frames = set()

        log.MIMEHandler.window = self

    def __getattr__(self, key):
        if key in self._symbols:
            raise AttributeError(key)

        if key in ('__members__', '__methods__'):
            raise AttributeError(key)

        if key == 'constructor':
            return PyV8.JSClassConstructor(self.__class__)

        if key == 'prototype':
            return PyV8.JSClassPrototype(self.__class__)

        prop = self.__dict__.setdefault('__properties__', {}).get(key, None)

        if prop and isinstance(prop[0], collections.Callable):
            return prop[0]()

        if log.ThugOpts.Personality.isIE() and key.lower() in ('wscript', 'wsh', ):
            # Prevent _ActiveXObject loops
            super(Window, self).__setattr__("WScript", None)
            WScript = _ActiveXObject(self, "WScript.Shell")
            super(Window, self).__setattr__(key, WScript)
            super(Window, self).__setattr__("WScript", WScript)
            return WScript

        context = self.__class__.__dict__['context'].__get__(self, Window)

        try:
            self._symbols.add(key)
            symbol = context.eval(key)
        except:
            raise AttributeError(key)
        finally:
            self._symbols.discard(key)

        if isinstance(symbol, PyV8.JSFunction):
            _method = None

            if symbol in self._methods:
                _method = symbol.clone()

            if _method is None:
                _method = new.instancemethod(symbol, self, Window)
                # _method = symbol.__get__(self, Window)

            setattr(self, key, _method)
            context.locals[key] = _method
            return _method

        if isinstance(symbol, (six.string_types,
                               bool,
                               numbers.Number,
                               datetime.datetime,
                               PyV8.JSObject)):
            setattr(self, key, symbol)
            context.locals[key] = symbol
            return symbol

        raise AttributeError(key)

    @property
    def closed(self):
        return self._closed

    def close(self):
        self._closed = True

    @property
    def this(self):
        return self

    @property
    def window(self):
        return self

    @property
    def self(self):
        return self

    @property
    def top(self):
        return self

    @property
    def _document(self):
        return self.doc

    def _findAll(self, tags):
        return self.doc.doc.find_all(tags, recursive = True)

    @property
    def frames(self):
        """an array of all the frames (including iframes) in the current window"""
        for frame in self._findAll(['frame', 'iframe']):
            code = unicode(frame)

            if code in self._inner_frames:
                continue

            self._inner_frames.add(code)
            self._frames.add(Window(self.url, w3c.parseString(code), personality = log.ThugOpts.useragent))

        return HTMLCollection(self.doc, list(self._frames))

    @property
    def length(self):
        """the number of frames (including iframes) in a window"""
        return len(self._findAll(['frame', 'iframe']))

    @property
    def history(self):
        """the History object for the window"""
        return self._history

    def getLocation(self):
        """the Location object for the window"""
        return self._location

    def setLocation(self, location):
        self._location.href = location

    location = property(getLocation, setLocation)

    @property
    def navigator(self):
        """the Navigator object for the window"""
        return self._navigator

    @property
    def opener(self):
        """a reference to the window that created the window"""
        return self._opener

    @property
    def pageXOffset(self):
        return 0

    @property
    def pageYOffset(self):
        return 0

    @property
    def parent(self):
        return self._parent

    @property
    def screen(self):
        return self._screen

    @property
    def screenLeft(self):
        return self._left

    @property
    def screenTop(self):
        return self._top

    @property
    def screenX(self):
        return self._left

    @property
    def screenY(self):
        return self._top

    def _do_ActiveXObject(self, cls, typename = 'name'):
        return _ActiveXObject(self, cls, typename)

    # Window object methods
    #
    # escape        Encodes a string.
    # sizeToContent Sizes the window according to its content.
    # unescape      Unencodes a value that has been encoded in hexadecimal (e.g., a cookie).

    def alert(self, text):
        """
        Display an alert dialog with the specified text.
        Syntax

        window.alert(text)

        Parameters

        text is a string of the text you want displayed in the alert dialog.
        """
        log.warning('[Window] Alert Text: %s', str(text))

    def back(self):
        """
        Returns the window to the previous item in the history.
        Syntax

        window.back()

        Parameters

        None.
        """
        pass

    def blur(self):
        """
        Shifts focus away from the window.
        Syntax

        window.blur()

        Parameters

        None.
        """
        pass

    def captureEvents(self, eventType):
        """
        Registers the window to capture all events of the specified type.
        Syntax

        window.captureEvents(Event.eventType)

        Parameters

        eventType is a string
        """
        self.alert("[Captured Event] %s" % (eventType, ))

    def clearInterval(self, intervalID):
        """
        Clears a delay that's been set for a specific function.
        Syntax

        window.clearInterval(intervalID)

        Parameters

        intervalID is the ID of the specific interval you want to clear.
        """
        self.timers[intervalID].stop()

    def clearTimeout(self, timeoutID):
        """
        Clears the delay set by window.setTimeout().
        Syntax

        window.clearTimeout(timeoutID)

        Parameters

        timeoutID is the ID of the timeout you wish you clear.
        """
        self.timers[timeoutID].stop()

    def confirm(self, text):
        """
        Displays a dialog with a message that the user needs to respond to.
        Syntax

        result = window.confirm(text)

        Parameters

        text is a string.

        result is a boolean value indicating whether OK or Cancel was selected.
        """
        return True

    def dump(self, text):
        """
        Prints messages to the console.
        Syntax

        window.dump(text)

        Parameters

        text is a string.
        """
        self.alert(text)

    def focus(self):
        """
        Sets focus on the window.
        Syntax

        window.focus()

        Parameters

        None.
        """
        pass

    def forward(self):
        """
        Moves the window one document forward in the history.
        Syntax

        window.forward()

        Parameters

        None.
        """
        self._history.forward()

    def GetAttention(self):
        """
        Flashes the application icon to get the user's attention.
        Syntax

        window.GetAttention()

        Parameters

        None.
        """
        pass

    def getSelection(self):
        """
        Returns the selection (generally text).
        Syntax

        selection = window.getSelection()

        Parameters

        selection is a selection object.
        """
        return None

    def home(self):
        """
        Returns the window to the home page.
        Syntax

        window.home()

        Parameters

        None.
        """
        self.open()

    def moveBy(self, deltaX, deltaY):
        """
        Moves the current window by a specified amount.
        Syntax

        window.moveBy(deltaX, deltaY)

        Parameters

        deltaX is the amount of pixels to move the window horizontally.
        deltaY is the amount of pixels to move the window vertically.
        """
        pass

    def moveTo(self, x, y):
        """
        Moves the window to the specified coordinates.
        Syntax

        window.moveTo(x, y)

        Parameters

        x is the horizontal coordinate to be moved to.
        y is the vertical coordinate to be moved to.
        """
        pass

    def prompt(self, text):
        """
        Returns the text entered by the user in a prompt dialog.
        """
        return text

    def releaseEvents(self, eventType):
        """
        Releases the window from trapping events of a specific type.
        Syntax

        window.releaseEvents(Event.eventType)

        Parameters

        eventType is a string
        """
        self.alert("[Released Event] %s" % (eventType, ))

    def resizeBy(self, xDelta, yDelta):
        """
        Resizes the current window by a certain amount.
        Syntax

        window.resizeBy(xDelta, yDelta)

        Parameters

        xDelta is the number of pixels to grow the window horizontally.
        yDelta is the number of pixels to grow the window vertically.
        """
        pass

    def resizeTo(self, iWidth, iHeight):
        """
        Dynamically resizes window.
        Syntax

        window.resizeTo(iWidth, iHeight)

        Parameters

        iWidth is an integer representing the new width in pixels.
        iHeight is an integer value representing the new height in pixels.
        """
        pass

    def scroll(self, x, y):
        """
        Scrolls the window to a particular place in the document.
        Syntax

        window.scroll(x-coord, y-coord)

        Parameters

        x-coord is the pixel along the horizontal axis of the document that
        you want displayed in the upper left.
        y-coord is the pixel along the vertical axis of the document that you
        want displayed in the upper left.
        """
        pass

    def scrollBy(self, xDelta, yDelta):
        """
        Scrolls the document in the window by the given amount.
        Syntax

        window.scrollBy(xDelta, yDelta)

        Parameters

        xDelta is the amount of pixels to scroll horizontally.

        yDelta is the amount of pixels to scroll vertically.
        """
        pass

    def scrollByLines(self, lines):
        """
        Scrolls the document by the given number of lines.
        Syntax

        window.scrollByLines(lines)

        Parameters

        lines is the number of lines.
        """
        pass

    def scrollByPages(self, pages):
        """
        Scrolls the current document by the specified number of pages.
        Syntax

        window.scrollByPages(pages)

        Parameters

        pages is the number of pages to scroll.
        """
        pass

    def scrollTo(self, x, y):
        """
        Scrolls to a particular set of coordinates in the document.
        Syntax

        window.scrollTo(x-coord, y-coord)

        Parameters

        x-coord is the pixel along the horizontal axis of the document that you
        want displayed in the upper left.

        y-coord is the pixel along the vertical axis of the document that you
        want displayed in the upper left.
        """
        pass

    def setCursor(self, s):
        pass

    def setInterval(self, f, delay, lang = 'JavaScript'):
        """
        Set a delay for a specific function.
        Syntax

        ID = window.setInterval("funcName", delay)

        Parameters

        funcName is the name of the function for which you want to set a
        delay.

        delay is the number of milliseconds (thousandths of a second)
        that the function should be delayed.

        ID is the interval ID.
        """
        if log.ThugOpts.delay:
            delay = min(delay, log.ThugOpts.delay)

        timer = Window.Timer(self, f, delay, True, lang)
        self.timers.append(timer)
        timer.start()

        return len(self.timers) - 1

    def setTimeout(self, f, delay = 0, lang = 'JavaScript'):
        """
        Sets a delay for executing a function.
        Syntax

        ID = window.setTimeout("funcName", delay)

        Parameters

        funcName is the name of the function for which you want to set a
        delay.

        delay is the number of milliseconds (thousandths of a second)
        that the function should be delayed.

        ID is the interval ID.
        """
        if log.ThugOpts.delay:
            delay = min(delay, log.ThugOpts.delay)

        timer = Window.Timer(self, f, delay, False, lang)
        self.timers.append(timer)
        timer.start()

        return len(self.timers) - 1

    def stop(self):
        """
        This method stops window loading.
        Syntax

        window.stop()

        Parameters

        None.
        """
        pass

    def _attachEvent(self, sEvent, fpNotify, useCapture = False):
        setattr(self, sEvent.lower(), fpNotify)

    def _detachEvent(self, sEvent, fpNotify):
        notify = getattr(self, sEvent.lower(), None)
        if notify is None:
            return

        if notify in (fpNotify, ):
            delattr(self, sEvent.lower())

    def _addEventListener(self, _type, listener, useCapture = False):
        setattr(self, 'on%s' % (_type.lower(), ), listener)

    def _removeEventListener(self, _type, listener, useCapture = False):
        _listener = getattr(self, 'on%s' % (_type.lower(), ), None)
        if _listener is None:
            return

        if _listener in (listener, ):
            delattr(self, 'on%s' % (_type.lower(), ))

    def _CollectGarbage(self):
        pass

    def _navigate(self, location):
        self.location = location
        return 0

    def _execScript(self, code, language = "JScript"):
        if log.ThugOpts.code_logging:
            log.ThugLogging.add_code_snippet(code, language, 'Contained_Inside')

        if language in ("JScript", ):
            self.eval(code)
        else:
            log.HTMLClassifier.classify(log.ThugLogging.url if log.ThugOpts.local else self.url, code)

        return None

    def __init_personality(self):
        if log.ThugOpts.Personality.isIE():
            self.__init_personality_IE()
            return

        if log.ThugOpts.Personality.isFirefox():
            self.__init_personality_Firefox()
            return

        if log.ThugOpts.Personality.isChrome():
            self.__init_personality_Chrome()
            return

        if log.ThugOpts.Personality.isSafari():
            self.__init_personality_Safari()
            return

        if log.ThugOpts.Personality.isOpera():
            self.__init_personality_Opera()

    def __init_personality_IE(self):
        if not (log.ThugOpts.local and log.ThugOpts.attachment):
            self.document       = self._document
            self.XMLHttpRequest = self._XMLHttpRequest

        self.ActiveXObject     = self._do_ActiveXObject
        self.CollectGarbage    = self._CollectGarbage
        self.navigate          = self._navigate
        self.clientInformation = self.navigator
        self.clipboardData     = ClipboardData()
        self.external          = External()

        self.ScriptEngineMajorVersion = log.ThugOpts.Personality.ScriptEngineMajorVersion
        self.ScriptEngineMinorVersion = log.ThugOpts.Personality.ScriptEngineMinorVersion
        self.ScriptEngineBuildVersion = log.ThugOpts.Personality.ScriptEngineBuildVersion

        if log.ThugOpts.Personality.browserMajorVersion < 11:
            self.execScript = self._execScript

        if log.ThugOpts.Personality.browserMajorVersion < 9:
            self.attachEvent = self._attachEvent
            self.detachEvent = self._detachEvent
        else:
            self.addEventListener    = self._addEventListener
            self.removeEventListener = self._removeEventListener

        if log.ThugOpts.Personality.browserMajorVersion >= 8:
            self.localStorage   = LocalStorage()
            self.sessionStorage = SessionStorage()

        self.doc.parentWindow = self._parent

    def __init_personality_Firefox(self):
        self.document            = self._document
        self.XMLHttpRequest      = self._XMLHttpRequest
        self.addEventListener    = self._addEventListener
        self.removeEventListener = self._removeEventListener
        self.crypto              = Crypto()
        self.sidebar             = Sidebar()
        self.Components          = Components()
        self.console             = Console()
        self.localStorage        = LocalStorage()
        self.sessionStorage      = SessionStorage()

        if log.ThugOpts.Personality.browserMajorVersion > 32:
            self.RadioNodeList = None

        if log.ThugOpts.Personality.browserMajorVersion > 12:
            self.Map = Map()

        if log.ThugOpts.Personality.browserMajorVersion > 11:
            self.navigator.mozConnection = mozConnection()

        with self.context as ctxt:
            if log.ThugOpts.Personality.browserMajorVersion <= 20:
                ctxt.eval("delete Math.imul;")
            if log.ThugOpts.Personality.browserMajorVersion <= 4:
                ctxt.eval("delete Array.isArray;")

    def __init_personality_Chrome(self):
        self.document            = self._document
        self.XMLHttpRequest      = self._XMLHttpRequest
        self.addEventListener    = self._addEventListener
        self.removeEventListener = self._removeEventListener
        self.clientInformation   = self.navigator
        self.external            = External()
        self.chrome              = Chrome()
        self.console             = Console()
        self.localStorage        = LocalStorage()
        self.sessionStorage      = SessionStorage()
        self.onmousewheel        = None

    def __init_personality_Safari(self):
        self.document            = self._document
        self.XMLHttpRequest      = self._XMLHttpRequest
        self.addEventListener    = self._addEventListener
        self.removeEventListener = self._removeEventListener
        self.clientInformation   = self.navigator
        self.console             = Console()
        self.localStorage        = LocalStorage()
        self.sessionStorage      = SessionStorage()
        self.onmousewheel        = None

    def __init_personality_Opera(self):
        self.document            = self._document
        self.XMLHttpRequest      = self._XMLHttpRequest
        self.addEventListener    = self._addEventListener
        self.removeEventListener = self._removeEventListener
        self.opera               = Opera()
        self.doc.parentWindow    = self._parent
        self.console             = Console()
        self.localStorage        = LocalStorage()
        self.sessionStorage      = SessionStorage()

    def eval(self, script):
        if script is None:
            return

        if len(script) > 64:
            log.warning("[Window] Eval argument length > 64 (%d)", len(script))

        if log.ThugOpts.code_logging and len(script) > 4:
            log.ThugLogging.add_code_snippet(script, 'Javascript', 'Dynamically_Evaluated', True)

        return self.evalScript(script)

    @property
    def context(self):
        # if not hasattr(self, '_context'):
        if '_context' not in self.__dict__:
            self._context = PyV8.JSContext(self, extensions = log.JSExtensions)
            with self._context as ctxt:
                thug_js = os.path.join(thug.__configuration_path__, 'scripts', "thug.js")
                ctxt.eval(open(thug_js, 'r').read())

                if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion < 8:
                    storage_js = os.path.join(thug.__configuration_path__, 'scripts', "storage.js")
                    ctxt.eval(open(storage_js, 'r').read())

                hooks_folder = os.path.join(thug.__configuration_path__, 'hooks')
                for hook in sorted([h for h in os.listdir(hooks_folder) if h.endswith('.js')]):
                    ctxt.eval(open(os.path.join(hooks_folder, hook), 'r').read())

                PyV8.JSEngine.collect()

        return self._context

    def evalScript(self, script, tag = None):
        if log.ThugOpts.verbose or log.ThugOpts.debug:
            log.info(script)

        result = 0

        try:
            log.JSClassifier.classify(log.ThugLogging.url if log.ThugOpts.local else self.url, script)

            if log.ThugOpts.code_logging:
                log.ThugLogging.add_code_snippet(script, 'Javascript', 'Contained_Inside')
        except:  # pylint:disable=bare-except
            pass

        if tag:
            self.doc.current = tag
        else:
            try:
                body = self.doc.body
            except:  # pylint:disable=bare-except
                # This code is for when you are desperate :)
                body = self.doc.getElementsByTagName('body')[0]

            if body and body.tag.contents:
                self.doc.current = body.tag.contents[-1]
            else:
                self.doc.current = self.doc.doc.contents[-1]

        with self.context as ctxt:
            try:
                ast = AST(self, script)
            except:  # pylint:disable=bare-except
                # log.debug(traceback.format_exc())
                return result

            if log.ThugOpts.Personality.isIE():
                cc = CCInterpreter()
                script = cc.run(script)

            shellcode = Shellcode.Shellcode(self, ctxt, ast, script)
            result    = shellcode.run()

        return result

    def unescape(self, s):
        i  = 0
        sc = list()

        if len(s) > 16:
            log.ThugLogging.shellcodes.add(s)

        # %xx format
        if '%' in s and '%u' not in s:
            return urllib.unquote(s)

        # %uxxxx format
        while i < len(s):
            if s[i] == '"':
                i += 1
                continue

            if s[i] == '%' and (i + 1) < len(s) and s[i + 1] == 'u':
                if (i + 6) <= len(s):
                    currchar = int(s[i + 2: i + 4], 16)
                    nextchar = int(s[i + 4: i + 6], 16)
                    sc.append(chr(nextchar))
                    sc.append(chr(currchar))
                    i += 6
                elif (i + 3) <= len(s):
                    currchar = int(s[i + 2: i + 4], 16)
                    sc.append(chr(currchar))
                    i += 3
            else:
                sc.append(s[i])
                i += 1

        return ''.join(sc)

    def Image(self, width = 800, height = 600):
        return self.doc.createElement('img')

    def _XMLHttpRequest(self):
        return _ActiveXObject(self, 'microsoft.xmlhttp')

    def getComputedStyle(self, element, pseudoelt = None):
        return getattr(element, 'style', None)

    def open(self, url = None, name = '_blank', specs = '', replace = False):
        if url:
            try:
                response = self._navigator.fetch(url, redirect_type = "window open")
            except:  # pylint:disable=bare-except
                # traceback.print_exc()
                return None

            if response is None:
                return None

            if response.status_code == 404:
                return None

            html = response.content

            try:
                log.HTMLClassifier.classify(log.ThugLogging.url if log.ThugOpts.local else url, html)
            except:  # pylint:disable=bare-except
                pass

            if response.history:
                url = response.url

            content_type = response.headers.get('content-type' , None)
            if content_type:
                handler = log.MIMEHandler.get_handler(content_type)
                if handler and handler(url, html):
                    return None

            # Log response here
            kwds = { 'referer' : self.url }
            if 'set-cookie' in response.headers:
                kwds['cookie'] = response.headers['set-cookie']
            if 'last-modified' in response.headers:
                kwds['lastModified'] = response.headers['last-modified']
        else:
            url  = 'about:blank'
            html = ''
            kwds = {}

        dom = BeautifulSoup.BeautifulSoup(html, "html5lib")

        for spec in specs.split(','):
            spec = [s.strip() for s in spec.split('=')]

            if len(spec) == 2:
                if spec[0] in ['width', 'height', 'left', 'top']:
                    kwds[spec[0]] = int(spec[1])

            if name in ['_blank', '_parent', '_self', '_top']:
                kwds['target'] = name
                name = ''
            else:
                kwds['target'] = '_blank'

        return Window(url, dom, navigator = None, personality = self._personality,
                        name = name, parent = self, opener = self, replace = replace, **kwds)



#!/usr/bin/env python

from .Node import Node


class ProcessingInstruction(Node):
    def __init__(self, doc, target, data):
        self._target = target
        self.data    = data
        Node.__init__(self, doc)

    @property
    def target(self):
        return self._target

    @property
    def nodeName(self):
        return self._target

    @property
    def nodeType(self):
        return Node.PROCESSING_INSTRUCTION_NODE

    def getNodeValue(self):
        return self.data

    def setNodeValue(self, data):
        self.data = data

    nodeValue = property(getNodeValue, setNodeValue)



# MacrovisionJob, MacrovisionFlexNet
# CVE-2007-2419, CVE-2007-5660, CVE-2007-6654, CVE-2007-0321, CVE-2007-0328

import logging

log = logging.getLogger("Thug")


def Initialize(self, *args):
    log.ThugLogging.add_behavior_warn('[Macrovision ActiveX] Initialize')


def CreateJob(self, name, arg, job_id):
    log.ThugLogging.add_behavior_warn('[Macrovision ActiveX] CreateObject("%s", "%s", "%s")' % (name, arg, job_id, ))
    return self


def DownloadAndExecute(self, arg0, arg1, arg2, arg3, arg4):
    log.ThugLogging.add_behavior_warn('[Macrovision ActiveX] DownloadAndExecute("%s", "%s", "%s", "%s", "%s")' % (arg0, arg1, arg2, arg3, arg4))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Macrovision ActiveX",
                                      "DownloadAndExecute",
                                      data = {
                                                "arg" : arg0,
                                                "arg1": arg1,
                                                "arg2": arg2,
                                                "arg3": arg3,
                                                "arg4": arg4
                                             },
                                      forward = False)

    if len(arg1) > 512:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Macrovision ActiveX",
                                          "DownloadAndExecute overflow",
                                          cve = "CVE-2007-2419, CVE-2007-6654")

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-2419", None)
    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6654", None)

    log.ThugLogging.add_behavior_warn("[Macrovision ActiveX] Fetching from URL %s" % (arg3, ))

    try:
        self._window._navigator.fetch(arg3, redirect_type = "Macrovision Exploit")
    except:  # pylint:disable=bare-except
        log.ThugLogging.add_behavior_warn('[Macrovision ActiveX] Fetch failed')


def DownloadAndInstall(self, *args):
    log.ThugLogging.add_behavior_warn('[Macrovision ActiveX] DownloadAndInstall')


def AddFileEx(self, arg0, arg1, arg2, arg3, arg4, arg5, arg6):
    if len(arg2) > 512:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Macrovision ActiveX",
                                          "AddFileEx overflow",
                                          cve = "CVE-2007-2419")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-2419", None)

def AddFile(self, arg0, arg1):
    log.ThugLogging.add_behavior_warn('[Macrovision ActiveX] AddFile("%s", "%s")' % (arg0, arg1))
    log.ThugLogging.add_behavior_warn("[Macrovision ActiveX] Fetching from URL %s" % (arg0, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Macrovision ActiveX",
                                      "AddFile/Fetch from URL",
                                      cve = "CVE-2007-2419",
                                      forward = False,
                                      data = {
                                                "url": arg0,
                                                "arg1": arg1
                                             }
                                     )

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-2419", None)

    try:
        self._window._navigator.fetch(arg0, redirect_type = "Macrovision Exploit 2")
    except:  # pylint:disable=bare-except
        log.ThugLogging.add_behavior_warn('[Macrovision ActiveX] Fetch failed')


def SetPriority(self, priority):
    log.ThugLogging.add_behavior_warn('[Macrovision ActiveX] SetPriority(%s)' % (priority, ))


def SetNotifyFlags(self, flags):
    log.ThugLogging.add_behavior_warn('[Macrovision ActiveX] SetNotifyFlags(%s)' % (flags, ))


def RunScheduledJobs(self):
    log.ThugLogging.add_behavior_warn('[Macrovision ActiveX] RunScheduledJobs()')



#!/usr/bin/env python

import logging

log = logging.getLogger("Thug")

import bs4 as BeautifulSoup
from .Node import Node
from .NodeList import NodeList


class DocumentFragment(Node):
    def __init__(self, doc):
        self.tag = BeautifulSoup.Tag(parser = doc, name = 'documentfragment')
        Node.__init__(self, doc)
        self.__init_personality()

    def __init_personality(self):
        if log.ThugOpts.Personality.isIE():
            self.__init_personality_IE()
            return

        if log.ThugOpts.Personality.isFirefox():
            self.__init_personality_Firefox()
            return

        if log.ThugOpts.Personality.isChrome():
            self.__init_personality_Chrome()
            return

        if log.ThugOpts.Personality.isSafari():
            self.__init_personality_Safari()
            return

        if log.ThugOpts.Personality.isOpera():
            self.__init_personality_Opera()

    def __init_personality_IE(self):
        if log.ThugOpts.Personality.browserMajorVersion > 7:
            self.querySelectorAll = self._querySelectorAll
            self.querySelector    = self._querySelector

    def __init_personality_Firefox(self):
        self.querySelectorAll = self._querySelectorAll
        self.querySelector    = self._querySelector

    def __init_personality_Chrome(self):
        self.querySelectorAll = self._querySelectorAll
        self.querySelector    = self._querySelector

    def __init_personality_Safari(self):
        self.querySelectorAll = self._querySelectorAll
        self.querySelector    = self._querySelector

    def __init_personality_Opera(self):
        self.querySelectorAll = self._querySelectorAll
        self.querySelector    = self._querySelector

    def _querySelectorAll(self, selectors):
        try:
            s = self.tag.select(selectors)
        except:  # pylint:disable=bare-except
            return NodeList(self.doc, [])

        return NodeList(self.doc, s)

    def _querySelector(self, selectors):
        from .DOMImplementation import DOMImplementation

        try:
            s = self.tag.select(selectors)
        except:  # pylint:disable=bare-except
            return None

        if s and s[0]:
            return DOMImplementation.createHTMLElement(self, s[0])

        return None

    @property
    def nodeName(self):
        return "#document-fragment"

    @property
    def nodeType(self):
        return Node.DOCUMENT_FRAGMENT_NODE

    @property
    def nodeValue(self):
        return None



# Microsoft XMLHTTP

import logging

try:
    import urllib.parse as urlparse
except ImportError:
    import urlparse

import thug.DOM as DOM

log = logging.getLogger("Thug")


def abort(self):
    log.ThugLogging.add_behavior_warn("[Microsoft XMLHTTP ActiveX] abort")
    return 0


def open(self, bstrMethod, bstrUrl, varAsync = True, varUser = None, varPassword = None):  # pylint:disable=redefined-builtin
    # Internet Explorer ignores any \r\n or %0d%0a or whitespace appended to the domain name
    parsedUrl = urlparse.urlparse(bstrUrl)
    netloc = parsedUrl.netloc.strip("\r\n\t")
    bstrUrl = urlparse.urlunparse((parsedUrl.scheme, netloc, parsedUrl.path, parsedUrl.params, parsedUrl.query, parsedUrl.fragment))

    msg = "[Microsoft XMLHTTP ActiveX] open('%s', '%s', %s" % (bstrMethod, bstrUrl, varAsync is True, )
    if varUser:
        msg = "%s, '%s'" % (msg, varUser, )
    if varPassword:
        msg = "%s, '%s'" % (msg, varPassword, )
    msg = "%s)" % (msg, )
    log.ThugLogging.add_behavior_warn(msg)
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Microsoft XMLHTTP ActiveX",
                                      "Open",
                                      forward = False,
                                      data = {
                                                "method" : str(bstrMethod),
                                                "url"    : str(bstrUrl),
                                                "async"  : str(varAsync)
                                             }
                                     )

    self.bstrMethod  = str(bstrMethod)
    self.bstrUrl     = str(bstrUrl)
    self.varAsync    = varAsync
    self.varUser     = varUser
    self.varPassword = varPassword
    self.readyState  = 4

    if self.onreadystatechange:
        with self._window.context as ctx:  # pylint:disable=unused-variable
            self.onreadystatechange.__call__()

    return 0


def send(self, varBody = None):
    msg = "send"
    if varBody:
        msg = "%s('%s')" % (msg, str(varBody), )

    log.ThugLogging.add_behavior_warn("[Microsoft XMLHTTP ActiveX] %s" % (msg, ))
    log.ThugLogging.add_behavior_warn("[Microsoft XMLHTTP ActiveX] Fetching from URL %s (method: %s)" % (self.bstrUrl, self.bstrMethod, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Microsoft XMLHTTP ActiveX",
                                      "Send",
                                      forward = False,
                                      data = {
                                                "method" : self.bstrMethod,
                                                "url"    : str(self.bstrUrl)
                                             }
                                     )

    response = None

    try:
        response = self._window._navigator.fetch(self.bstrUrl,
                                                 method        = self.bstrMethod,
                                                 headers       = self.requestHeaders,
                                                 body          = varBody,
                                                 redirect_type = "Microsoft XMLHTTP Exploit")
    except:  # pylint:disable=bare-except
        log.ThugLogging.add_behavior_warn('[Microsoft XMLHTTP ActiveX] Fetch failed')

    if response is None:
        return 0

    self.status          = response.status_code
    self.responseHeaders = response.headers
    self.responseBody    = response.content
    self.responseText    = response.content
    self.readyState      = 4

    contenttype = self.responseHeaders.get('content-type', None)
    if contenttype is None:
        return 0

    if 'text/html' in contenttype:
        doc = DOM.W3C.w3c.parseString(self.responseBody)

        window = DOM.Window.Window(self.bstrUrl, doc, personality = log.ThugOpts.useragent)
        # window.open(self.bstrUrl)

        dft = DOM.DFT.DFT(window)
        dft.run()
        return

    handler = log.MIMEHandler.get_handler(contenttype)
    if handler:
        handler(self.bstrUrl, self.responseBody)

    if self.onreadystatechange:
        with self._window.context as ctx: # pylint:disable=unused-variable
            self.onreadystatechange.__call__()

    return 0


def setRequestHeader(self, bstrHeader, bstrValue):
    log.ThugLogging.add_behavior_warn("[Microsoft XMLHTTP ActiveX] setRequestHeaders('%s', '%s')" % (bstrHeader, bstrValue, ))
    self.requestHeaders[bstrHeader] = bstrValue
    return 0


def getResponseHeader(self, header):
    body = ""
    if header in self.responseHeaders:
        body = self.responseHeaders[header]

    try:
        self._window._navigator.fetch(self.bstrUrl,
                                      method  = self.bstrMethod,
                                      headers = self.requestHeaders,
                                      body    = body)
    except:  # pylint:disable=bare-except
        pass


def getAllResponseHeaders(self):
    body = ""
    for k, v in self.responseHeaders.items():
        body += "%s: %s\r\n" % (k, v, )

    try:
        self._window._navigator.fetch(self.bstrUrl,
                                      method  = self.bstrMethod,
                                      headers = self.requestHeaders,
                                      body    = body)
    except:  # pylint:disable=bare-except
        pass


def overrideMimeType(self, mimetype):
    pass



#!/usr/bin/env python

import string
import logging
from .HTMLElement import HTMLElement

log = logging.getLogger("Thug")


class TAnimateColor(HTMLElement):
    def __init__(self, doc, tag):
        self.doc = doc
        self.tag = tag
        self._values = ""

    def get_values(self):
        return self._values

    def set_values(self, values):
        if all(c in string.printable for c in values) is False:
            log.ThugLogging.log_exploit_event(self.doc.window.url,
                                              "Microsoft Internet Explorer",
                                              "Microsoft Internet Explorer CButton Object Use-After-Free Vulnerability (CVE-2012-4792)",
                                              cve = 'CVE-2012-4792',
                                              forward = True)

            log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2012-4792", None)

        log.DFT.check_shellcode(values)
        self._values = values

    values = property(get_values, set_values)



#!/usr/bin/env python

from .Event import Event

import logging

log = logging.getLogger("Thug")


# Introduced in DOM Level 2
class StorageEvent(Event):
    StorageEventTypes = ('storage', )

    def __init__(self, eventTypeArg, target):
        Event.__init__(self, target)

    def initStorageEvent(self,
                         eventTypeArg,
                         canBubbleArg,
                         cancelableArg,
                         keyArg,
                         oldValueArg,
                         newValueArg,
                         urlArg,
                         storageAreaArg):
        self._type       = eventTypeArg
        self.typeArg     = eventTypeArg
        self._canBubble  = canBubbleArg
        self._cancelable = cancelableArg
        self.key         = keyArg
        self.oldValue    = oldValueArg
        self.newValue    = newValueArg
        self.url         = urlArg
        self.storageArea = storageAreaArg



#!/usr/bin/env python

from .Event import Event

import logging

log = logging.getLogger("Thug")


# Introduced in DOM Level 2
class UIEvent(Event):
    UIEventTypes = ('DOMFocusIn', 'DOMFocusOut', 'DOMActivate')

    def __init__(self, typeArg, target):
        Event.__init__(self, target)
        canBubbleArg  = typeArg in ('DOMFocusIn', 'DOMFocusOut', 'DOMActivate')
        cancelableArg = typeArg in ('DOMActivate', )
        self.initUIEvent(typeArg, canBubbleArg, cancelableArg)

    @property
    def view(self):
        return self._view

    @property
    def detail(self):
        return self._detail

    def initUIEvent(self, typeArg, canBubbleArg, cancelableArg, viewArg = None, detailArg = 0):
        log.debug('initUIEvent(%s, %s, %s, %s, %s)', typeArg,
                                                     canBubbleArg,
                                                     cancelableArg,
                                                     viewArg,
                                                     detailArg)

        self.initEvent(typeArg, canBubbleArg, cancelableArg)
        self._view   = viewArg
        self._detail = detailArg



#!/usr/bin/env python

import logging

import thug.DOM.W3C as W3C
from .HTMLElement import HTMLElement
from .attr_property import attr_property

log = logging.getLogger("Thug")


class HTMLIFrameElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)
        self.document = W3C.w3c.getDOMImplementation()

    align           = attr_property("align")
    frameBorder     = attr_property("frameborder")
    height          = attr_property("height")
    longDesc        = attr_property("longdesc")
    marginHeight    = attr_property("marginheight")
    marginWidth     = attr_property("marginwidth")
    name            = attr_property("name")
    scrolling       = attr_property("scrolling")
    src             = attr_property("src")
    width           = attr_property("width")

    # Introduced in DOM Level 2
    @property
    def contentDocument(self):
        return self.doc if self.doc else None

    @property
    def contentWindow(self):
        if self.id in log.ThugLogging.windows:
            return log.ThugLogging.windows[self.id]

        if self.doc is None:
            return None

        return getattr(self.doc, 'window', None)



#!/usr/bin/env python
#
# AST.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging
import PyV8
import traceback

log = logging.getLogger("Thug")


class AST(object):
    (AssignBreakPoint,
     LoopBreakPoint) = range(0, 2)

    AssignOps = [PyV8.AST.Op.ASSIGN,
                 PyV8.AST.Op.ASSIGN_ADD,
                 PyV8.AST.Op.ASSIGN_BIT_AND,
                 PyV8.AST.Op.ASSIGN_BIT_OR,
                 PyV8.AST.Op.ASSIGN_BIT_XOR,
                 PyV8.AST.Op.ASSIGN_DIV,
                 PyV8.AST.Op.ASSIGN_MOD,
                 PyV8.AST.Op.ASSIGN_MUL,
                 PyV8.AST.Op.ASSIGN_SAR,
                 PyV8.AST.Op.ASSIGN_SHL,
                 PyV8.AST.Op.ASSIGN_SHR,
                 PyV8.AST.Op.ASSIGN_SUB,
                 PyV8.AST.Op.INIT_VAR]

    def __init__(self, window, script):
        self.names           = set()
        self.inLoop          = False
        self.inBlock         = True
        self.exitingLoop     = 0
        self.assignStatement = False
        self.breakpoints     = set()
        self.window          = window

        self.walk(script)

    def checkExitingLoop(self, pos):
        if self.exitingLoop > 0:
            self.exitingLoop -= 1
            self.breakpoints.add((self.LoopBreakPoint, pos))

    def walk(self, script):
        self.block_no = 1

        try:
            PyV8.JSEngine().compile(script).visit(self)
        except UnicodeDecodeError:
            enc = log.Encoding.detect(script, safe = True)
            if enc is None:
                return

            PyV8.JSEngine().compile(script.decode(enc['encoding'])).visit(self)
        except:  # pylint:disable=bare-except
            pass

    def onProgram(self, prog):
        for decl in prog.scope.declarations:
            decl.visit(self)

        for stmt in prog.body:
            stmt.visit(self)

    def _enterBlock(self):
        self.inBlock = True

    def _exitBlock(self):
        self.inBlock = False

    def onBlock(self, block):
        self._enterBlock()
        for stmt in block.statements:
            stmt.visit(self)

        self._exitBlock()
        self.block_no += 1

    def onExpressionStatement(self, stmt):
        self.checkExitingLoop(stmt.expression.pos)
        stmt.expression.visit(self)
        if self.assignStatement:
            if self.inBlock:
                # FIXME
                # AstCallRuntime has no 'pos' attribute
                try:
                    pos = stmt.expression.pos
                except:  # pylint:disable=bare-except
                    traceback.print_exc()
                    return
            else:
                pos = stmt.expression.pos

            self.breakpoints.add((self.AssignBreakPoint, pos))
            self.assignStatement = False

    def onVariableDeclaration(self, decl):
        var = decl.proxy

        # if decl.scope.isGlobal:
        #    getattr(self.window, var.name, None)

        if decl.mode == PyV8.AST.VarMode.var:
            self.names.add(var.name)

    def onFunctionDeclaration(self, decl):
        # f = decl.proxy

        # if decl.scope.isGlobal:
        #    getattr(self.window, f.name, None)

        for d in decl.scope.declarations:
            if not getattr(d, 'function', None):
                continue

            d.function.visit(self)

            # for stmt in d.function.body:
            #    stmt.visit(self)

    def onAssignment(self, expr):
        if not self.inLoop:
            if expr.op in self.AssignOps:
                self.assignStatement = True

        self.names.add(str(expr.target))
        expr.target.visit(self)
        expr.value.visit(self)

    def onIfStatement(self, stmt):
        stmt.condition.visit(self)

        if stmt.hasThenStatement:
            stmt.thenStatement.visit(self)
        if stmt.hasElseStatement:
            stmt.elseStatement.visit(self)

    def enterLoop(self):
        self.inLoop = True

    def exitLoop(self):
        self.inLoop = False
        self.exitingLoop += 1

    def onForStatement(self, stmt):
        self.checkExitingLoop(stmt.pos)
        self.enterLoop()

        if stmt.init:
            stmt.init.visit(self)

        if stmt.nextStmt:
            stmt.nextStmt.visit(self)

        if stmt.condition:
            stmt.condition.visit(self)

        if stmt.body:
            stmt.body.visit(self)

        self.exitLoop()

    def onWhileStatement(self, stmt):
        self.checkExitingLoop(stmt.pos)
        self.enterLoop()
        stmt.condition.visit(self)
        stmt.body.visit(self)
        self.exitLoop()

    def onDoWhileStatement(self, stmt):
        self.checkExitingLoop(stmt.pos)
        self.enterLoop()
        stmt.condition.visit(self)
        stmt.body.visit(self)
        self.exitLoop()

    def onForInStatement(self, stmt):
        self.checkExitingLoop(stmt.pos)
        self.enterLoop()
        stmt.enumerable.visit(self)
        stmt.body.visit(self)
        self.exitLoop()

    def onCall(self, expr):
        for arg in expr.args:
            arg.visit(self)

        handle = getattr(log.ASTHandler, "handle_%s" % (expr.expression, ), None)
        if handle:
            handle(expr.args)

        expr.expression.visit(self)

    def onCallNew(self, expr):
        handle = getattr(log.ASTHandler, "handle_%s" % (expr.expression, ), None)
        if handle:
            handle(expr.args)

        for arg in expr.args:
            arg.visit(self)

    def onCallRuntime(self, expr):
        for arg in expr.args:
            arg.visit(self)

    def onFunctionLiteral(self, litr):
        for decl in litr.scope.declarations:
            decl.visit(self)

        for e in litr.body:
            e.visit(self)

    def onLiteral(self, litr):
        if len(str(litr)) > 256:
            log.ThugLogging.shellcodes.add(str(litr).lstrip('"').rstrip('"'))

    def onReturnStatement(self, stmt):
        stmt.expression.visit(self)

    def onCompareOperation(self, stmt):
        stmt.left.visit(self)
        stmt.right.visit(self)

    def onCountOperation(self, stmt):
        stmt.expression.visit(self)



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property


class HTMLOptGroupElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    disabled        = attr_property("disabled", bool)
    label           = attr_property("label")



# BitDefender Online Scanner ActiveX Control
# CVE-2007-5775

import logging

log = logging.getLogger("Thug")


def initx(self, arg):
    if len(arg) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "BitDefender Online Scanner ActiveX",
                                          "InitX overflow",
                                          cve = "CVE-2007-5775")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-5775", None)



# Ourgame GLWorld HanGamePluginCn18 Class ActiveX Control
# CVE-2008-0647

import logging

log = logging.getLogger("Thug")


def hgs_startGame(self, arg):
    if len(arg) > 1000:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Ourgame GLWorld ActiveX",
                                          "Overflow in hgs_startGame",
                                          cve = 'CVE-2008-0647')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2008-0647", None)

def hgs_startNotify(self, arg):
    if len(arg) > 1000:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Ourgame GLWorld ActiveX",
                                          "Overflow in hgs_startGame",
                                          cve = 'CVE-2008-0647')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2008-0647", None)



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property

import logging

log = logging.getLogger("Thug")


class HTMLFormElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    @property
    def elements(self):
        raise NotImplementedError()

    @property
    def length(self):
        raise NotImplementedError()

    name            = attr_property("name")
    acceptCharset   = attr_property("accept-charset", default = "UNKNOWN")
    action          = attr_property("action")
    enctype         = attr_property("enctype", default = "application/x-www-form-urlencoded")
    method          = attr_property("method", default = "get")
    target          = attr_property("target")

    def submit(self):
        handler = getattr(log.DFT, 'handle_form', None)
        if handler:
            handler(self.tag)

    def reset(self):
        log.warning('[HTMLFormElement] reset method not defined')



#!/usr/bin/env python
#
# Navigator.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


import os
import hashlib
import logging

from .JSClass import JSClass
from .MimeTypes import MimeTypes
from .Plugins import Plugins
from .UserProfile import UserProfile
from .HTTPSessionException import AboutBlank
from .HTTPSessionException import FetchForbidden
from .HTTPSessionException import InvalidUrl
from .HTTPSessionException import ThresholdExpired
from thug.Magic.Magic import Magic

log = logging.getLogger("Thug")


class Navigator(JSClass):
    def __init__(self, personality, window = None):
        self.personality = log.ThugOpts.Personality[personality]
        self._plugins    = Plugins()  # An array of the plugins installed in the browser
        self._mimeTypes  = MimeTypes()
        self._window     = window

        for p in self._mimeTypes.values():
            self._plugins.append(p['enabledPlugin'])

        self.__init_personality()
        self.filecount = 0

    def __init_personality(self):
        if log.ThugOpts.Personality.isIE():
            self.__init_personality_IE()
            return

        if log.ThugOpts.Personality.isFirefox():
            self.__init_personality_Firefox()
            return

        if log.ThugOpts.Personality.isChrome():
            self.__init_personality_Chrome()
            return

        if log.ThugOpts.Personality.isSafari():
            self.__init_personality_Safari()
            return

        if log.ThugOpts.Personality.isOpera():
            self.__init_personality_Opera()

    def __init_personality_IE(self):
        self.mimeTypes       = dict()
        self.plugins         = self._plugins
        self.taintEnabled    = self._taintEnabled
        self.appMinorVersion = self._appMinorVersion
        self.cpuClass        = self._cpuClass
        self.browserLanguage = self._browserLanguage
        self.systemLanguage  = self._systemLanguage
        self.userLanguage    = self._userLanguage

        if log.ThugOpts.Personality.browserMajorVersion < 9:
            self.userProfile = UserProfile()

    def __init_personality_Firefox(self):
        self.mimeTypes    = self._mimeTypes
        self.plugins      = self._plugins
        self.taintEnabled = self._taintEnabled
        self.oscpu        = self._oscpu
        self.buildID      = self._buildID
        self.product      = self._product
        self.productSub   = self._productSub
        self.vendor       = self._vendor
        self.vendorSub    = self._vendorSub
        self.language     = self._language
        self.preference   = self._preference

        self.registerContentHandler  = self._registerContentHandler
        self.registerProtocolHandler = self._registerProtocolHandler

    def __init_personality_Chrome(self):
        self.mimeTypes  = self._mimeTypes
        self.plugins    = self._plugins
        self.product    = self._product
        self.productSub = self._productSub
        self.vendor     = self._vendor
        self.vendorSub  = self._vendorSub
        self.language   = self._language

    def __init_personality_Safari(self):
        self.mimeTypes  = self._mimeTypes
        self.plugins    = self._plugins
        self.product    = self._product
        self.productSub = self._productSub
        self.vendor     = self._vendor
        self.vendorSub  = self._vendorSub
        self.language   = self._language

    def __init_personality_Opera(self):
        self.mimeTypes       = self._mimeTypes
        self.plugins         = self._plugins
        self.taintEnabled    = self._taintEnabled
        self.appMinorVersion = self._appMinorVersion
        self.browserLanguage = self._browserLanguage
        self.language        = self._language
        self.userLanguage    = self._userLanguage

    @property
    def window(self):
        return self._window

    @property
    def appCodeName(self):
        """
            The internal "code" name of the current browser
        """
        return self.personality['appCodeName']

    @property
    def appName(self):
        """
            The official name of the browser
        """
        return self.personality['appName']

    @property
    def appVersion(self):
        """
            The version of the browser as a string
        """
        return self.personality['appVersion']

    @property
    def userAgent(self):
        """
            The user agent string for the current browser
        """
        return self.personality['userAgent']

    @property
    def _buildID(self):
        """
            The build identifier of the browser (e.g. "2006090803")
        """
        return self.personality['buildID']

    @property
    def cookieEnabled(self):
        """
            A boolean indicating whether cookies are enabled
        """
        return True

    @property
    def _language(self):
        """
            A string representing the language version of the browser
        """
        return "en"

    @property
    def onLine(self):
        """
            A boolean indicating whether the browser is working online
        """
        return True

    @property
    def _oscpu(self):
        """
            A string that represents the current operating system
        """
        return self.personality['oscpu']

    @property
    def platform(self):
        """
            A string representing the platform of the browser
        """
        return self.personality['platform']

    @property
    def _product(self):
        """
            The product name of the current browser (e.g. "Gecko")
        """
        return self.personality['product']

    @property
    def _productSub(self):
        """
            The build number of the current browser (e.g. "20060909")
        """
        return self.personality['productSub']

    @property
    def securityPolicy(self):
        """
            An empty string. In Netscape 4.7x, returns "US & CA domestic policy" or "Export policy".
        """
        return ""

    @property
    def _vendor(self):
        """
            The vendor name of the current browser (e.g. "Netscape6")
        """
        return self.personality['vendor']

    @property
    def _vendorSub(self):
        """
            The vendor name of the current browser (e.g. "Netscape6")
        """
        return self.personality['vendorSub']

    @property
    def _appMinorVersion(self):
        return self.personality['appMinorVersion']

    @property
    def _browserLanguage(self):
        return "en"

    @property
    def _cpuClass(self):
        return "x86"

    @property
    def _systemLanguage(self):
        return "en"

    @property
    def _userLanguage(self):
        return "en"

    # Indicates whether the host browser is Java-enabled or not.
    def javaEnabled(self, *arg):
        return True

    # Lets code check to see if the document at a given URI is
    # available without using the network.
    def mozIsLocallyAvailable(self, *arg):
        return False

    # Sets a user preference.
    # self method is only available to privileged code, and you
    # should use XPCOM Preferences API instead.
    def _preference(self, *arg):
        pass

    # Allows web sites to register themselves as a possible handler
    # for a given MIME type.
    def _registerContentHandler(self, *arg):
        pass

    # New in Firefox 3
    # Allows web sites to register themselves as a possible handler
    # for a given protocol.
    def _registerProtocolHandler(self, *arg):
        pass

    # Obsolete
    # JavaScript taint/untaint functions removed in JavaScript 1.2[1]
    def _taintEnabled(self, *arg):
        return True

    def fetch(self, url, method = "GET", headers = None, body = None, redirect_type = None, params = None):
        log.URLClassifier.classify(url)

        # The command-line option -x (--local-nofetch) prevents remote
        # content fetching so raise an exception and exit the method.
        if log.HTTPSession.no_fetch:
            raise FetchForbidden

        # Do not attempt to fetch content if the URL is "about:blank".
        if log.HTTPSession.about_blank(url):
            raise AboutBlank

        # URL normalization and fixing (if broken and the option is
        # enabled).
        url = log.HTTPSession.normalize_url(self._window, url)
        if url is None:
            raise InvalidUrl

        if redirect_type:
            log.ThugLogging.add_behavior_warn(("[%s redirection] %s -> %s" % (redirect_type, self._window.url, url, )))
            log.ThugLogging.log_connection(self._window.url, url, redirect_type)
        else:
            log.ThugLogging.log_connection(self._window.url, url, "unknown")

        # The command-line option -t (--threshold) defines the maximum
        # number of pages to fetch. If the threshold is reached avoid
        # fetching the contents.
        if log.HTTPSession.threshold_expired(url):
            raise ThresholdExpired

        if headers is None:
            headers = dict()

        response = log.HTTPSession.fetch(url, method, self._window, self.userAgent, headers, body)
        if response is None:
            return None

        _url = log.ThugLogging.log_redirect(response)
        if _url:
            url = _url

        referer = response.request.headers.get('referer', 'None')
        log.ThugLogging.add_behavior_warn("[HTTP] URL: %s (Status: %s, Referer: %s)" % (url, response.status_code, referer, ))

        ctype     = response.headers.get('content-type', 'unknown')
        mime_base = os.path.join(log.ThugLogging.baseDir, ctype)

        md5 = hashlib.md5()
        md5.update(response.content)
        sha256 = hashlib.sha256()
        sha256.update(response.content)

        mtype = Magic(response.content).get_mime()

        data = {
            "content" : response.content,
            "status"  : response.status_code,
            "md5"     : md5.hexdigest(),
            "sha256"  : sha256.hexdigest(),
            "fsize"   : len(response.content),
            "ctype"   : ctype,
            "mtype"   : mtype
        }

        log.ThugLogging.add_behavior_warn("[HTTP] URL: %s (Content-type: %s, MD5: %s)" % (response.url, ctype, data["md5"]))
        log.ThugLogging.log_location(url, data)

        if response.history:
            location = response.headers.get('location', None)
            if location and redirect_type not in ("URL found", "JNLP", "iframe", ):
                self._window.url = location

        if redirect_type in ("meta", ):
            self._window.url = url

        log.ThugLogging.store_content(mime_base, data["md5"], response.content)
        log.ThugLogging.log_file(response.content, response.url, params)
        return response



# Yahoo! Messenger 8.x Ywcvwr ActiveX Control
# CVE-2007-4391

import logging
log = logging.getLogger("Thug")


def Setserver(self, name):
    self.__dict__['server'] = name

    if len(name) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Yahoo! Messenger 8.x Ywcvwr ActiveX",
                                          "Server Console Overflow",
                                          cve = "CVE-2007-4391")

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4391", None)


def GetComponentVersion(self, arg):
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Yahoo! Messenger 8.x Ywcvwr ActiveX",
                                      "GetComponentVersion Overflow",
                                      cve = "CVE-2007-4391")

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4391", None)


def initialize(self):
    return


def send(self):
    return


def receive(self):
    return



# Xunlei DPClient.Vod.1 ActiveX Control DownURL2 Method Remote Buffer Overflow Vulnerability
# CVE-2007-5064

import logging

log = logging.getLogger("Thug")


def DownURL2(self, arg0, *args):
    if len(arg0) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Xunlei DPClient.Vod.1 ActiveX",
                                          "DownURL2 Method Buffer Overflow",
                                          cve = "CVE-2007-5064")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-5064", None)



# RTSP MPEG4 SP Control 1.x
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def SetMP4Prefix(self, val):
    self.__dict__['MP4Prefix'] = val

    if len(val) > 128:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "RTSP MPEG4 SP Control ActiveX",
                                          "Overflow in MP4Prefix property")



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .compatibility import thug_long


class HTMLTableCellElement(HTMLElement):
    def __init__(self, doc, tag, index = 0):
        HTMLElement.__init__(self, doc, tag)
        self._cellIndex = index

    @property
    def cellIndex(self):
        return self._cellIndex

    abbr            = attr_property("abbr")
    align           = attr_property("align")
    axis            = attr_property("axis")
    bgColor         = attr_property("bgcolor")
    ch              = attr_property("char")
    chOff           = attr_property("charoff")
    colSpan         = attr_property("colspan", thug_long)
    headers         = attr_property("headers")
    height          = attr_property("height")
    noWrap          = attr_property("nowrap", bool)
    rowSpan         = attr_property("rowspan", thug_long)
    scope           = attr_property("scope")
    vAlign          = attr_property("valign")
    width           = attr_property("width")



#!/usr/bin/env python

from .Event import Event


# Introduced in DOM Level 2
class MutationEvent(Event):
    MODIFICATION = 1  # The Attr was just added
    ADDITION     = 2  # The Attr was modified in place
    REMOVAL      = 3  # The Attr was just removed

    @property
    def relatedNode(self):
        return None

    @property
    def prevValue(self):
        return None

    @property
    def newValue(self):
        return None

    @property
    def attrName(self):
        return None

    @property
    def attrChange(self):
        return None

    def initMutationEvent(self, typeArg,
                                canBubbleArg,
                                cancelableArg,
                                relatedNodeArg,
                                prevValueArg,
                                newValueArg,
                                attrNameArg,
                                attrChangeArg):
        pass



#!/usr/bin/env python

from .CSSStyleDeclaration import CSSStyleDeclaration


class ElementCSSInlineStyle(object):
    def __init__(self, doc, tag):
        self.doc = doc
        self.tag = tag

    @property
    def style(self):
        return CSSStyleDeclaration(self.tag['style'] if self.tag.has_attr('style') else '')



#!/usr/bin/env python
#
# ASTHandler.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging

log = logging.getLogger("Thug")


class ASTHandler(object):
    def __init__(self):
        self.args = {
            'eval' : list(),
        }

    def handle_eval(self, args):
        for arg in args:
            s = str(arg)

            if s in self.args['eval']:
                continue

            if len(s) > 64:
                log.warning("[AST]: Eval argument length > 64")

            self.args['eval'].append(s)



#!/usr/bin/env python

from .Event import Event

import logging

log = logging.getLogger("Thug")


# Introduced in DOM Level 2
class HTMLEvent(Event):
    HTMLEventTypes = ('load',
                      'unload',
                      'abort',
                      'error',
                      'select',
                      'change',
                      'submit',
                      'reset',
                      'focus',
                      'blur',
                      'resize',
                      'scroll')

    def __init__(self, eventTypeArg, target):
        Event.__init__(self, target)
        canBubbleArg  = eventTypeArg in ('abort', 'error', 'select', 'change', 'submit', 'reset', 'resize', 'scroll')
        cancelableArg = eventTypeArg in ('submit', )
        self.initEvent(eventTypeArg, canBubbleArg, cancelableArg)

    def initEvent(self, eventTypeArg, canBubbleArg, cancelableArg):
        self._type       = eventTypeArg
        self._canBubble  = canBubbleArg
        self._cancelable = cancelableArg



#!/usr/bin/env python

from thug.DOM.W3C.abstractmethod import abstractmethod
from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .compatibility import thug_long
from .compatibility import thug_maxint


class HTMLInputElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    # defaultValue    = attr_property("value")
    value           = attr_property("value")
    defaultChecked  = attr_property("checked", bool)

    @property
    def form(self):
        raise NotImplementedError()

    accept          = attr_property("accept")
    accessKey       = attr_property("accesskey")
    align           = attr_property("align")
    alt             = attr_property("alt")
    checked         = attr_property("checked", bool)
    disabled        = attr_property("disabled", bool)
    maxLength       = attr_property("maxlength", thug_long, default = thug_maxint)
    name            = attr_property("name")
    readOnly        = attr_property("readonly", bool)
    size            = attr_property("size", thug_long)
    src             = attr_property("src")
    tabIndex        = attr_property("tabindex", thug_long)
    type            = attr_property("type", default = "text")
    useMap          = attr_property("usermap")

    @abstractmethod
    def getValue(self):
        pass

    @abstractmethod
    def setValue(self, value):
        pass

    # value = property(getValue, setValue)

    def blur(self):
        pass

    def focus(self):
        pass

    def select(self):
        pass

    def click(self):
        pass



from thug.ActiveX.modules import WScriptShell
import logging
import random
import string

log = logging.getLogger("Thug")


class WshCollection(list):
    def __getattr__(self, name):
        if name.lower() == 'length':
            return len(self)

    def Item(self, pos):
        return self[pos]


def EnumPrinterConnections(self):
    log.ThugLogging.add_behavior_warn("[WScript.Network ActiveX] Got request to PrinterConnections")

    printerlist = [['nul:', 'Send To OneNote 2010'],
                   ['XPSPort:', 'Microsoft XPS Document Writer'],
                   ['SHRFAX:', 'Fax']]

    for _ in range(3):
        ip = GetRandomIp()
        printerlist.append(['IP_{}'.format(ip), GetRandomShare(ip)])

    random.shuffle(printerlist)
    return WshCollection(sum(printerlist[:2], []))


def EnumNetworkDrives(self):
    log.ThugLogging.add_behavior_warn("[WScript.Network ActiveX] Got request to EnumNetworkDrives")
    ndrives = WshCollection()

    for _ in range(2):
        drive = "{}:".format(chr(random.choice(range(ord('E'), ord('Z')))))
        ndrives.extend([drive, GetRandomShare(GetRandomIp())])

    return ndrives


def GetRandomShare(location):
    share = "".join(random.choice(string.ascii_lowercase + string.digits) for _ in range(8))
    return "\\\\{}\\{}".format(location, share)


def GetRandomIp():
    ip = "192.168."
    ip += ".".join(map(str, (random.randint(0, 255) for _ in range(2))))
    return ip


def GetUserDomain(self):
    return WScriptShell.ExpandEnvironmentStrings(self, "%USERDOMAIN%")


def GetUserName(self):
    return WScriptShell.ExpandEnvironmentStrings(self, "%USERNAME%")


def GetComputerName(self):
    return WScriptShell.ExpandEnvironmentStrings(self, "%COMPUTERNAME%")



# IBM Lotus Domino Web Access Control ActiveX Control
# CVE-2007-4474

import logging

log = logging.getLogger("Thug")


def SetGeneral_ServerName(self, val):
    self.__dict__['General_ServerName'] = val

    if len(val) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "IBM Lotus Domino Web Access Control ActiveX",
                                          "Overflow in General_ServerName property",
                                          cve = 'CVE-2007-4474')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4474", None)


def SetGeneral_JunctionName(self, val):
    self.__dict__['General_JunctionName'] = val

    if len(val) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "IBM Lotus Domino Web Access Control ActiveX",
                                          "Overflow in General_JunctionName property",
                                          cve = 'CVE-2007-4474')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4474", None)


def SetMail_MailDbPath(self, val):
    self.__dict__['Mail_MailDbPath'] = val

    if len(val) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "IBM Lotus Domino Web Access Control ActiveX",
                                          "Overflow in Mail_MailDbPath property",
                                          cve = 'CVE-2007-4474')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4474", None)


def InstallBrowserHelperDll(self):
    pass



# GOM Player GOM Manager ActiveX Control
# CVE-2007-5779

import logging

log = logging.getLogger("Thug")


def OpenURL(self, arg):
    if len(arg) > 500:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "GOM Player Manager ActiveX",
                                          "Overflow in OpenURL",
                                          cve = "CVE-2007-5779")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-5779", None)



import logging

log = logging.getLogger("Thug")


def save(self):
    log.ThugLogging.add_behavior_warn("[WScript.Shortcut ActiveX] Saving link object '{}' with target '{}'".format(self.FullName, self.TargetPath))
    return



from thug.ActiveX.modules import WScriptShell
from thug.ActiveX.modules import TextStream
from thug.ActiveX.modules import File
from thug.OS.Windows import win32_files

import os
import string
import random
import logging

log = logging.getLogger("Thug")


def OpenTextFile(self, sFilePathAndName, ForWriting = True, flag = True):
    log.ThugLogging.add_behavior_warn('[Script.FileSystemObject ActiveX] OpenTextFile("%s", "%s", "%s")' % (sFilePathAndName, ForWriting, flag))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Script.FileSystemObject ActiveX",
                                      "OpenTextFile",
                                      data = {
                                                "filename"  : sFilePathAndName,
                                                "ForWriting": ForWriting,
                                                "flag"      : flag
                                             },
                                      forward = False)


def Write(self, sFileContents):
    log.ThugLogging.add_behavior_warn('[Script.FileSystemObject ActiveX] Write("%s")' % (sFileContents, ))


def Close(self):
    log.ThugLogging.add_behavior_warn('[Script.FileSystemObject ActiveX] Close()')


def BuildPath(self, arg0, arg1):
    log.ThugLogging.add_behavior_warn('[Script.FileSystemObject ActiveX] BuildPath("%s", "%s")' % (arg0, arg1, ))
    return "%s\%s" % (arg0, arg1)


def GetSpecialFolder(self, arg):
    log.ThugLogging.add_behavior_warn('[Script.FileSystemObject ActiveX] GetSpecialFolder(%s)' % (arg, ))

    arg = int(arg)
    folder = ''
    if arg == 0:
        folder = WScriptShell.ExpandEnvironmentStrings(self, "%windir%")
    elif arg == 1:
        folder = WScriptShell.ExpandEnvironmentStrings(self, "%SystemRoot%\\system32")
    elif arg == 2:
        folder = WScriptShell.ExpandEnvironmentStrings(self, "%TEMP%")

    log.ThugLogging.add_behavior_warn('[Script.FileSystemObject ActiveX] Returning %s for GetSpecialFolder(%s)' % (folder, arg, ))
    return folder


def GetTempName(self):
    log.ThugLogging.add_behavior_warn('[Script.FileSystemObject ActiveX] GetTempName()')
    return ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(8))


def FileExists(self, filespec):
    log.ThugLogging.add_behavior_warn('[Script.FileSystemObject ActiveX] FileExists(%s)' % (filespec, ))
    if filespec.lower() in win32_files:
        return True

    return False


def CreateTextFile(self, filename, overwrite = False, _unicode = False):
    log.ThugLogging.add_behavior_warn('[Script.FileSystemObject ActiveX] CreateTextFile(%s, %s, %s)' % (filename, overwrite, _unicode))
    stream = TextStream.TextStream()
    stream._filename = filename
    return stream


def GetFile(self, filespec):
    log.ThugLogging.add_behavior_warn('[Script.FileSystemObject ActiveX] GetFile(%s)' % (filespec, ))
    return File.File(filespec)


def GetExtensionName(self, path):
    log.ThugLogging.add_behavior_warn('[Script.FileSystemObject ActiveX] GetExtensionName(%s)' % (path, ))
    name, ext = os.path.splitext(path)
    return ext if ext else ""



#!/usr/bin/env python
#
# ThugAPI.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import sys
import os
import logging
import PyV8

from bs4 import BeautifulSoup
from zope.interface import implementer
from lxml.html import builder as E
from lxml.html import tostring

try:
    import urllib.parse as urlparse
except ImportError:
    import urlparse

import thug
from thug.DOM.W3C import w3c
from thug.DOM.Window import Window
from thug.DOM.HTTPSession import HTTPSession
from thug.DOM.DFT import DFT
from thug.DOM.MIMEHandler import MIMEHandler
from thug.DOM.SchemeHandler import SchemeHandler
from thug.WebTracking.WebTracking import WebTracking
from thug.Encoding.Encoding import Encoding
from thug.Logging.ThugLogging import ThugLogging
from thug.AST.ASTHandler import ASTHandler

from .IThugAPI import IThugAPI
from .ThugOpts import ThugOpts
from .ThugVulnModules import ThugVulnModules
from .OpaqueFilter import OpaqueFilter
from .Watchdog import Watchdog
from .abstractmethod import abstractmethod

from thug.Classifier.HTMLClassifier import HTMLClassifier
from thug.Classifier.JSClassifier import JSClassifier
from thug.Classifier.VBSClassifier import VBSClassifier
from thug.Classifier.URLClassifier import URLClassifier
from thug.Classifier.SampleClassifier import SampleClassifier
from thug.Classifier.TextClassifier import TextClassifier

log = logging.getLogger("Thug")
log.setLevel(logging.WARN)


@implementer(IThugAPI)
class ThugAPI(object):
    def __init__(self, configuration_path = thug.__configuration_path__):
        log.configuration_path  = configuration_path
        log.personalities_path  = os.path.join(configuration_path, "personalities") if configuration_path else None
        log.ThugOpts            = ThugOpts()
        log.ThugVulnModules     = ThugVulnModules()
        log.WebTracking         = WebTracking()
        log.MIMEHandler         = MIMEHandler()
        log.SchemeHandler       = SchemeHandler()
        log.HTMLClassifier      = HTMLClassifier()
        log.JSClassifier        = JSClassifier()
        log.VBSClassifier       = VBSClassifier()
        log.URLClassifier       = URLClassifier()
        log.SampleClassifier    = SampleClassifier()
        log.TextClassifier      = TextClassifier()
        log.Encoding            = Encoding()
        log.ASTHandler          = ASTHandler()
        log.JSExtensions        = list()
        log.Trace               = None

    def __call__(self):
        self.analyze()

    def usage(self):
        pass

    def version(self):
        print("Thug %s" % (thug.__version__, ))
        sys.exit(0)

    def get_useragent(self):
        return log.ThugOpts.useragent

    def set_useragent(self, useragent):
        log.ThugOpts.useragent = useragent

    def get_events(self):
        return log.ThugOpts.events

    def set_events(self, events):
        log.ThugOpts.events = events

    def get_delay(self):
        return log.ThugOpts.delay

    def set_delay(self, delay):
        log.ThugOpts.delay = delay

    def get_attachment(self):
        return log.ThugOpts.attachment

    def set_attachment(self):
        log.ThugOpts.attachment = True

    def get_file_logging(self):
        return log.ThugOpts.file_logging

    def set_file_logging(self):
        log.ThugOpts.file_logging = True

    def get_json_logging(self):
        return log.ThugOpts.json_logging

    def set_json_logging(self):
        log.ThugOpts.json_logging = True

    def get_maec11_logging(self):
        return log.ThugOpts.maec11_logging

    def set_maec11_logging(self):
        log.ThugOpts.maec11_logging = True

    def get_elasticsearch_logging(self):
        return log.ThugOpts.elasticsearch_logging

    def set_elasticsearch_logging(self):
        log.ThugOpts.elasticsearch_logging = True
        logging.getLogger("elasticsearch").setLevel(logging.ERROR)

    def get_referer(self):
        return log.ThugOpts.referer

    def set_referer(self, referer):
        log.ThugOpts.referer = referer

    def get_proxy(self):
        return log.ThugOpts.proxy

    def set_proxy(self, proxy):
        log.ThugOpts.proxy = proxy

    def get_raise_for_proxy(self):
        return log.ThugOpts.raise_for_proxy

    def set_raise_for_proxy(self, raise_for_proxy):
        log.ThugOpts.raise_for_proxy = raise_for_proxy

    def set_no_fetch(self):
        log.ThugOpts.no_fetch = True

    def set_verbose(self):
        log.ThugOpts.verbose = True
        log.setLevel(logging.INFO)

    def set_debug(self):
        log.ThugOpts.debug = True
        log.setLevel(logging.DEBUG)

    def set_ast_debug(self):
        log.ThugOpts.ast_debug = True

    def set_http_debug(self):
        log.ThugOpts.http_debug += 1

    def set_acropdf_pdf(self, acropdf_pdf):
        log.ThugVulnModules.acropdf_pdf = acropdf_pdf

    def disable_acropdf(self):
        log.ThugVulnModules.disable_acropdf()

    def set_shockwave_flash(self, shockwave):
        log.ThugVulnModules.shockwave_flash = shockwave

    def disable_shockwave_flash(self):
        log.ThugVulnModules.disable_shockwave_flash()

    def set_javaplugin(self, javaplugin):
        log.ThugVulnModules.javaplugin = javaplugin

    def disable_javaplugin(self):
        log.ThugVulnModules.disable_javaplugin()

    def get_threshold(self):
        return log.ThugOpts.threshold

    def set_threshold(self, threshold):
        log.ThugOpts.threshold = threshold

    def get_extensive(self):
        return log.ThugOpts.extensive

    def set_extensive(self):
        log.ThugOpts.extensive = True

    def get_timeout(self):
        return log.ThugOpts.timeout

    def set_timeout(self, timeout):
        log.ThugOpts.timeout = timeout

    def get_connect_timeout(self):
        return log.ThugOpts.connect_timeout

    def set_connect_timeout(self, timeout):
        log.ThugOpts.connect_timeout = timeout

    def get_broken_url(self):
        return log.ThugOpts.broken_url

    def set_broken_url(self):
        log.ThugOpts.broken_url = True

    def get_web_tracking(self):
        return log.ThugOpts.web_tracking

    def set_web_tracking(self):
        log.ThugOpts.web_tracking = True

    def disable_honeyagent(self):
        log.ThugOpts.honeyagent = False

    def enable_code_logging(self):
        log.ThugOpts.code_logging = True

    def disable_code_logging(self):
        log.ThugOpts.code_logging = False

    def enable_cert_logging(self):
        log.ThugOpts.cert_logging = True

    def disable_cert_logging(self):
        log.ThugOpts.cert_logging = False

    def log_init(self, url):
        log.ThugLogging = ThugLogging(thug.__version__)
        log.ThugLogging.set_basedir(url)

    def set_log_dir(self, logdir):
        log.ThugLogging.set_absbasedir(logdir)

    def set_log_output(self, output):
        fh = logging.FileHandler(output)
        log.addHandler(fh)

    def set_log_quiet(self):
        root = logging.getLogger()
        for handler in root.handlers:
            if isinstance(handler, logging.StreamHandler):
                handler.addFilter(OpaqueFilter())

    def set_vt_query(self):
        log.ThugOpts.set_vt_query()

    def set_vt_submit(self):
        log.ThugOpts.set_vt_submit()

    def get_vt_runtime_apikey(self):
        return log.ThugOpts.vt_runtime_apikey

    def set_vt_runtime_apikey(self, vt_runtime_apikey):
        log.ThugOpts.vt_runtime_apikey = vt_runtime_apikey

    def get_mongodb_address(self):
        return log.ThugOpts.mongodb_address

    def set_mongodb_address(self, mongodb_address):
        log.ThugOpts.mongodb_address = mongodb_address

    def add_htmlclassifier(self, rule):
        log.HTMLClassifier.add_rule(rule)

    def add_urlclassifier(self, rule):
        log.URLClassifier.add_rule(rule)

    def add_jsclassifier(self, rule):
        log.JSClassifier.add_rule(rule)

    def add_vbsclassifier(self, rule):
        log.VBSClassifier.add_rule(rule)

    def add_textclassifier(self, rule):
        log.TextClassifier.add_rule(rule)

    def add_sampleclassifier(self, rule):
        log.SampleClassifier.add_rule(rule)

    def add_htmlfilter(self, f):
        log.HTMLClassifier.add_filter(f)

    def add_urlfilter(self, f):
        log.URLClassifier.add_filter(f)

    def add_jsfilter(self, f):
        log.JSClassifier.add_filter(f)

    def add_vbsfilter(self, f):
        log.VBSClassifier.add_filter(f)

    def add_textfilter(self, f):
        log.TextClassifier.add_filter(f)

    def add_samplefilter(self, f):
        log.SampleClassifier.add_filter(f)

    def log_event(self):
        log.ThugLogging.log_event()

    def watchdog_cb(self, signum, frame):
        pass

    def __run(self, window):
        if log.Trace:
            sys.settrace(log.Trace)

        with PyV8.JSLocker():
            with Watchdog(log.ThugOpts.timeout, callback = self.watchdog_cb):
                dft = DFT(window)
                dft.run()

    def run_local(self, url):
        log.ThugLogging.set_url(url)
        log.ThugOpts.local = True

        log.HTTPSession = HTTPSession()

        content   = open(url, 'r').read()
        extension = os.path.splitext(url)

        if len(extension) > 1 and extension[1].lower() in ('.js', '.jse', ):
            if not content.lstrip().startswith('<script'):
                html = tostring(E.HTML(E.HEAD(), E.BODY(E.SCRIPT(content))))
            else:
                soup = BeautifulSoup(content, "html.parser")

                try:
                    soup.html.unwrap()
                except AttributeError:
                    pass

                try:
                    soup.head.unwrap()
                except AttributeError:
                    pass

                try:
                    soup.body.unwrap()
                except AttributeError:
                    pass

                html = tostring(E.HTML(E.HEAD(), E.BODY(E.SCRIPT(soup.script.get_text()))))
        else:
            html = content

        doc    = w3c.parseString(html)
        window = Window('about:blank', doc, personality = log.ThugOpts.useragent)
        window.open()
        self.__run(window)

    def run_remote(self, url):
        scheme = urlparse.urlparse(url).scheme

        if not scheme or not scheme.startswith('http'):
            url = 'http://%s' % (url, )

        log.ThugLogging.set_url(url)

        log.HTTPSession = HTTPSession()

        doc    = w3c.parseString('')
        window = Window(log.ThugOpts.referer, doc, personality = log.ThugOpts.useragent)
        window = window.open(url)
        if window:
            self.__run(window)

    @abstractmethod
    def analyze(self):
        pass



#!/usr/bin/env python
#
# UserProfile.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

from .JSClass import JSClass


class UserProfile(JSClass):
    vCardSchemas = ("vCard.Business.City",
                    "vCard.Business.Country",
                    "vCard.Business.Fax",
                    "vCard.Business.Phone",
                    "vCard.Business.State",
                    "vCard.Business.StreetAddress",
                    "vCard.Business.URL",
                    "vCard.Business.Zipcode",
                    "vCard.Cellular",
                    "vCard.Company",
                    "vCard.Department",
                    "vCard.DisplayName",
                    "vCard.Email",
                    "vCard.FirstName",
                    "vCard.Gender",
                    "vCard.Home.City",
                    "vCard.Home.Country",
                    "vCard.Home.Fax",
                    "vCard.Home.Phone",
                    "vCard.Home.State",
                    "vCard.Home.StreetAddress",
                    "vCard.Home.Zipcode",
                    "vCard.Homepage",
                    "vCard.JobTitle",
                    "vCard.LastName",
                    "vCard.MiddleName",
                    "vCard.Notes",
                    "vCard.Office",
                    "vCard.Pager")

    def __init__(self):
        self._vCard = dict()
        self._queue = list()

    def addReadRequest(self, vCardName, reserved = None):
        for schema in self.vCardSchemas:
            if schema.lower() == vCardName.lower():
                self.queue.append(vCardName)
                return True

        return False

    def doReadRequest(self, usageCode,
                            displayName = None,
                            domain = None,
                            path = None,
                            expiration = None,
                            reserved = None):
        pass

    def clearRequest(self):
        del self._queue[:]

    def getAttribute(self, vCardName):
        if vCardName not in self.vCardSchemas:
            return None

        if vCardName not in self._vCard:
            return None

        return self._vCard[vCardName]

    def setAttribute(self, vCardName, vCardValue, caseSens = 1):
        if caseSens:
            if vCardName not in self.vCardSchemas:
                return

            self._vCard[vCardName] = vCardValue
            return

        for schema in self.vCardSchemas:
            if schema.lower() == vCardName.lower():
                self._vCard[schema] = vCardValue
                return



#!/usr/bin/env python
#
# Cookies.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import datetime
import logging

log = logging.getLogger("Thug")

MAX_COOKIE_EXPIRES_DAYS = 365


class Cookies(object):
    now = datetime.datetime.now()
    cookie_expires_delta = datetime.timedelta(days = MAX_COOKIE_EXPIRES_DAYS)

    def __init__(self):
        self.cookie_id = 1
        self.cookies = set()

    def _inspect_cookie_expires(self, cookie):
        if not cookie.expires:
            return

        expires = datetime.datetime.fromtimestamp(cookie.expires)
        if self.now + self.cookie_expires_delta < expires:
            log.ThugLogging.log_warning("[TRACKING] [Cookie #{}] Expiring at {} (more than {} days from now)".format(self.cookie_id,
                                                                                                                     expires,
                                                                                                                     MAX_COOKIE_EXPIRES_DAYS))

        if self.now > expires:
            log.ThugLogging.log_warning("[TRACKING] [Cookie #{}] Expired at {}".format(self.cookie_id, expires))

    def _inspect_cookie_domain_initial_dot(self, cookie):
        if cookie.domain_specified and cookie.domain_initial_dot:
            log.ThugLogging.log_warning("[TRACKING] [Cookie #{}] Domain starting with initial dot: {}".format(self.cookie_id,
                                                                                                              cookie.domain))

    def _inspect_cookie_path(self, cookie):
        if cookie.path_specified:
            log.ThugLogging.log_warning("[TRACKING] [Cookie #{}] Path: {}".format(self.cookie_id, cookie.path))

    def _inspect_cookie_port(self, cookie):
        if cookie.port_specified:
            log.ThugLogging.log_warning("[TRACKING] [Cookie #{}] Port: {}".format(self.cookie_id, cookie.port))

    def _inspect_cookie_secure(self, cookie):
        if cookie.secure:
            log.ThugLogging.log_warning("[TRACKING] [Cookie #{}] Secure flag set".format(self.cookie_id))

    def _do_inspect_cookies(self, response):
        for cookie in response.cookies:
            self.cookies.add(cookie)
            log.ThugLogging.log_warning("[TRACKING] [Cookie #{}] {}".format(self.cookie_id, cookie.value))

            self._inspect_cookie_expires(cookie)
            self._inspect_cookie_domain_initial_dot(cookie)
            self._inspect_cookie_path(cookie)
            self._inspect_cookie_port(cookie)
            self._inspect_cookie_secure(cookie)
            self.cookie_id += 1

    def inspect(self, response):
        if response.history:
            for r in response.history:
                self._do_inspect_cookies(r)

        self._do_inspect_cookies(response)



import logging

log = logging.getLogger("Thug")


def isVersionSupported(self, version):
    shockwave = log.ThugVulnModules.shockwave_flash.split('.')
    sversion  = version.split('?')

    if len(sversion) == 1:
        sversion = version.split('.')

    if len(sversion) != 4:
        return False

    for i in range(0, 4):
        if int(sversion[i]) > int(shockwave[i]):
            return False

    return True



#!/usr/bin/env python
#
# Alexa.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import random

AlexaTopSites = [
        'http://www.google.com',
        'http://www.youtube.com',
        'http://www.facebook.com',
        'http://www.baidu.com',
        'http://www.wikipedia.org',
        'http://www.yahoo.com',
        'http://www.reddit.com',
        'http://www.taobao.com',
        'http://www.amazon.com',
        'http://www.tmall.com',
        'http://www.sohu.com',
        'http://www.twitter.com',
        'http://www.live.com',
        'http://www.instagram.com',
        'http://www.jd.com',
        'http://www.linkedin.com',
        'http://www.netflix.com',
        'http://www.t.co',
        'http://www.imgur.com',
        'http://www.ebay.com',
        'http://www.pornhub.com',
        'http://www.detail.tmall.com',
        'http://www.wordpress.com',
        'http://www.msn.com',
        'http://www.bing.com',
        'http://www.tumblr.com',
        'http://www.microsoft.com',
        'http://www.stackoverflow.com',
        'http://www.twitch.tv',
        'http://www.imdb.com',
        'http://www.blogspot.com',
        'http://www.office.com',
        'http://www.github.com',
        'http://www.microsoftonline.com',
        'http://www.apple.com',
        'http://www.popads.net',
        'http://www.diply.com',
        'http://www.pinterest.com',
        'http://www.csdn.net',
        'http://www.paypal.com',
        'http://www.adobe.com',
        'http://www.whatsapp.com',
        'http://www.xvideos.com',
        'http://www.xhamster.com',
        'http://www.pixnet.net',
        'http://www.login.tmall.com',
        'http://www.soso.com',
        'http://www.coccoc.com',
        'http://www.txxx.com',
        'http://www.dropbox.com',
        'http://www.googleusercontent.com',
        'http://www.bbc.co.uk',
]

random.shuffle(AlexaTopSites)
Alexa = AlexaTopSites[:random.randint(5, 9)]



# NeoTraceExplorer.NeoTraceLoader ActiveX control (NeoTraceExplorer.dll)
# CVE-2006-6707

import logging

log = logging.getLogger("Thug")


def TraceTarget(self, target):
    if len(target) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "NeoTraceExplorer.NeoTraceLoader ActiveX",
                                          "Overflow in arg0",
                                          cve = 'CVE-2006-6707')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2006-6707", None)



#!/usr/bin/env python
#
# Debugger.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import os
import logging
import PyV8

log = logging.getLogger("Thug")

import thug


class Debugger(PyV8.JSDebugger):
    def __init__(self):
        PyV8.JSDebugger.__init__(self)
        # self.evalContext = PyV8.JSContext()

    def __enter__(self):
        script_filename = os.path.join(thug.__configuration_path__, 'scripts', 'd8.js')
        with self.context as ctxt:
            ctxt.eval(open(script_filename, 'r').read())

        PyV8.JSEngine.collect()
        # self.setEnabled(True)

        return PyV8.JSDebugger.__enter__(self)

    def onMessage(self, msg):
        log.debug("Debug message: %s", msg)

        if msg['type'] == 'event' and msg['event'] == 'break':
            self.stepNext()
        return True

    def onDebugEvent(self, _type, state, evt):
        json = evt.toJSONProtocol()
        log.debug("%s event: %s", _type, json)

    def onBreakEvent(self, evt):
        log.debug("Break event: %s", evt)

    def onException(self, evt):
        log.debug("Exception event: %s", evt)

    def onNewFunction(self, evt):
        log.debug("New function event: %s", evt)

    def onBeforeCompile(self, evt):
        log.debug("Before compile event: %s", evt)
        return True

    def onAfterCompile(self, evt):
        log.debug("After compile event: %s", evt)
        return True

    def processDebugEvent(self, evt):
        log.debug("Received debug event: %s", repr(evt))

    def onBreak(self):
        log.debug("onBreak")



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property


class HTMLLinkElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    disabled        = False
    charset         = attr_property("charset", default = "")
    href            = attr_property("href")
    hreflang        = attr_property("hreflang")
    media           = attr_property("media")
    rel             = attr_property("rel")
    rev             = attr_property("rev")
    target          = attr_property("target")
    type            = attr_property("type")



# Qvod Player QvodCtrl Class ActiveX Control
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def SetURL(self, val):
    self.__dict__['URL'] = val
    self.__dict__['url'] = val

    if len(val) > 800:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Qvod Player QvodCtrl Class ActiveX",
                                          "Overflow in URL property")



#!/usr/bin/env python
#
# IThugAPI.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import zope.interface

class IThugAPI(zope.interface.Interface):
    def version():
        """
        Print Thug version and exit

        @return: None
        """

    def get_useragent():
        """
        get_useragent

        Return the emulated user agent

        @return: user agent string
        """

    def set_useragent(useragent):
        """
        set_useragent

        Set the user agent to emulate

        @param useragent: the user agent to emulate
        @type useragent: C{str}
        @return: None
        """

    def get_events():
        """
        get_events

        Return the DOM events to emulate
        Note: the load and mousemove are emulated by default and are not included in
        the returned list

        @return: List of the DOM events to emulate
        """

    def set_events(events):
        """
        set_events

        Set the DOM events to emulate
        Note: the load and mousemove events are emulated by default and do not
        need to be added through set_events

        @param events: comma separated list of DOM events to emulate
        @type events: C{str}
        @return: None
        """

    def get_delay():
        """
        get_delay

        Return the maximum setTimeout/setInterval delay value (in milliseconds)

        @return: maximum delay value (in milliseconds)
        """

    def set_delay(delay):
        """
        set_delay

        Set a maximum setTimeout/setInterval delay value (in milliseconds)

        @param delay: maximum delay value (in milliseconds)
        @type delay: C{int}
        @return: None
        """

    def get_attachment():
        """
        get_attachment

        Return True if the attachment mode is set, False otherwise

        @return: boolean
        """

    def set_attachment(attachment):
        """
        set_attachment

        Set the attachment mode to the specified value

        @param attachment: enable/disable attachment mode
        @type delay: C{bool}
        @return: None
        """

    def get_file_logging():
        """
        get_file_logging

        Return True if file logging mode is enabled, False otherwise.

        @return: boolean
        """

    def set_file_logging():
        """
        set_file_logging

        Enable file logging mode

        @return: None
        """

    def get_json_logging():
        """
        get_json_logging

        Return True if JSON logging mode is enabled, False otherwise.

        @return: boolean
        """

    def set_json_logging():
        """
        set_JSON_logging

        Enable JSON logging mode

        @return: None
        """

    def get_maec11_logging():
        """
        get_maec11_logging

        Return True if MAEC 1.1 logging mode is enabled, False otherwise.

        @return: boolean
        """

    def set_maec11_logging():
        """
        set_maec11_logging

        Enable MAEC 1.1 logging mode

        @return: None
        """

    def get_elasticsearch_logging():
        """
        get_elasticsearch_logging

        Return True if ElasticSearch logging mode is enabled, False otherwise.

        @return: boolean
        """

    def set_elasticsearch_logging():
        """
        set_elasticsearch_logging

        Enable ElasticSearch logging mode

        @return: None
        """

    def get_referer():
        """
        get_referer

        Return the emulated referer

        @return: referer value
        """

    def set_referer(referer):
        """
        set_referer

        Set the referer to be emulated

        @param referer: referer
        @type referer: C{str}
        @return: None
        """

    def get_proxy():
        """
        get_proxy

        Get the proxy server to be used for establishing the connection

        @return: proxy server
        """

    def set_proxy(proxy):
        """
        set_proxy

        Set the proxy server to be used for establishing the connection

        @param proxy: proxy server
        @type proxy: C{str}
        @return: None
        """

    def get_raise_for_proxy():
        """
        get_raise_for_proxy

        Get the raise_for_proxy flag. If the flag is True (default) a ValueError exception
        is raised if the specified proxy is not available.

        @return: boolean
        """

    def set_raise_for_proxy(raise_for_proxy):
        """
        set_raise_for_proxy

        Set the raise_for_proxy flag. If the flag is True (default) a ValueError exception
        is raised if the specified proxy is not available.

        @param raise_for_proxy: raise_for_proxy flag
        @type: raise_for_proxy: boolean
        @return: None
        """

    def set_no_fetch():
        """
        set_no_fetch

        Prevent remote content fetching in any case

        @return: None
        """

    def set_verbose():
        """
        set_verbose

        Enable Thug verbose mode

        @return: None
        """

    def set_debug():
        """
        set_debug

        Enable Thug debug mode

        @return: None
        """

    def set_ast_debug():
        """
        set_ast_debug

        Enable Thug AST debug mode

        @return: None
        """

    def set_http_debug():
        """
        set_http_debug

        Enable Thug HTTP debug mode

        @return: None
        """

    def set_acropdf_pdf(acropdf_pdf):
        """
        set_acropdf_pdf

        Set the Adobe Acrobat Reader version

        @param acropdf_pdf: Adobe Acrobat Reader version
        @type acropdf_pdf: C{str}
        @return: None
        """

    def disable_acropdf():
        """
        disable_acropdf

        Disable Adobe Acrobat Reader

        @return: None
        """

    def set_shockwave_flash(shockwave):
        """
        set_shockwave_flash

        Set the Shockwave Flash version (supported versions: 8, 9, 10, 11, 12)

        @param shockwave: Shockwave Flash version
        @type shockwave: C{str}
        @return: None
        """

    def disable_shockwave_flash():
        """
        disable_shockwave_flash

        Disable Shockwave Flash

        @return: None
        """

    def set_javaplugin(javaplugin):
        """
        set_javaplugin

        Set the Java plugin version

        @param javaplugin: Java plugin version
        @type javaplugin: C{str}
        @return: None
        """

    def disable_javaplugin():
        """
        disable_javaplugin

        Disable Java plugin

        @return: None
        """

    def get_threshold():
        """
        get_threshold

        Get the maximum number of pages to fetch

        @return: the maximum number of pages to fetch
        """

    def set_threshold(threshold):
        """
        set_threshold

        Set the maximum number of pages to fetch

        @param threshold: the maximum number of pages to fetch
        @type threshold: C{int}
        @return: None
        """

    def get_extensive():
        """
        get_extensive

        Get the current extensive fetch of linked pages mode

        @return: None
        """

    def set_extensive():
        """
        set_extensive

        Set the extensive fetch of linked pages mode

        @return: None
        """

    def get_connect_timeout():
        """
        get_connect_timeout

        Get the connect timeout (in seconds)

        @return: the connect timeout (in seconds)
        """

    def set_connect_timeout(timeout):
        """
        set_connect_timeout

        Set the connect timeout (in seconds)

        @param timeout: the connect timeout (in seconds)
        @type timeout: C{int}
        @return: None
        """

    def get_timeout():
        """
        get_timeout

        Get the analysis timeout (in seconds)

        @return: the analysis timeout (in seconds)
        """

    def set_timeout(timeout):
        """
        set_timeout

        Set the analysis timeout (in seconds)

        @param timeout: the analysis timeout (in seconds)
        @type timeout: C{int}
        @return: None
        """

    def get_broken_url():
        """
        get_broken_url

        Get the broken URL mode

        @return mode: broken URL mode
        """

    def set_broken_url():
        """
        set_broken_url

        Set the broken URL mode

        @return: None
        """

    def disable_honeyagent():
        """
        disable_honeyagent

        Disable HoneyAgent Java sandbox analysis

        @return: None
        """

    def enable_code_logging():
        """
        enable_code_logging

        Enable code logging

        @return: None
        """

    def disable_code_logging():
        """
        disable_code_logging

        Disable code logging

        @return: None
        """

    def enable_cert_logging():
        """
        enable_cert_logging

        Enable SSL/TLS certificate logging

        @return: None
        """

    def disable_cert_logging():
        """
        disable_cert_logging

        Disable SSL/TLS certificate logging

        @return: None
        """

    def log_init(url):
        """
        log_init

        Initialize logging subsystem

        @param url: URL to analyze
        @type url: C{str}
        @return: None
        """

    def set_log_dir(logdir):
        """
        set_log_dir

        Set the log output directory

        @param logdir: the log output directory
        @type logdir: C{str}
        @return: None
        """

    def set_log_output(output):
        """
        set_log_output

        Set the log output file

        @param output: the log output file
        @type output: C{str}
        @return: None
        """

    def set_log_quiet():
        """
        set_log_quiet

        Disable console logging

        @return: None
        """

    def set_vt_query():
        """
        set_vt_query

        Enable VirusTotal queries for sample analysis

        @return: None
        """

    def set_vt_submit():
        """
        set_vt_submit

        Enable VirusTotal samples submit

        @return: None
        """

    def get_web_tracking():
        """
        get_web_tracking

        Return True if web client tracking inspection is enabled, False otherwise.

        @return: bool
        """

    def set_web_tracking():
        """
        set_web_tracking

        Enable web client tracking inspection

        @return: None
        """

    def add_urlclassifier(rule):
        """
        add_urlclassifier

        Add an additional URL classifier rule file

        @param rule: URL classifier rule file
        @type rule: C{str}
        @return: None
        """

    def add_htmlclassifier(rule):
        """
        add_htmlclassifier

        Add an additional HTML classifier rule file

        @param rule: HTML classifier rule file
        @type rule: C{str}
        @return: None
        """

    def add_jsclassifier(rule):
        """
        add_jsclassifier

        Add an additional JS classifier rule file

        @param rule: JS classifier rule file
        @type rule: C{str}
        @return: None
        """

    def add_vbsclassifier(rule):
        """
        add_vbsclassifier

        Add an additional VBS classifier rule file

        @param rule: VBS classifier rule file
        @type rule: C{str}
        @return: None
        """

    def add_textclassifier(rule):
        """
        add_textclassifier

        Add an additional Text classifier rule file

        @param rule: Text classifier rule file
        @type rule: C{str}
        @return: None
        """

    def add_sampleclassifier(rule):
        """
        add_sampleclassifier

        Add an additional sample classifier rule file

        @param rule: Sample classifier rule file
        @type rule: C{str}
        @return: None
        """

    def add_urlfilter(filter):
        """
        add_urlfilter

        Add an additional URL filter file

        @param filter: URL filter file
        @type filter: C{str}
        @return: None
        """

    def add_htmlfilter(filter):
        """
        add_htmlfilter

        Add an additional HTML filter file

        @param filter: HTML filter file
        @type filter: C{str}
        @return: None
        """

    def add_jsfilter(filter):
        """
        add_jsfilter

        Add an additional JS filter file

        @param filter: JS filter file
        @type filter: C{str}
        @return: None
        """

    def add_vbsfilter(filter):
        """
        add_vbsfilter

        Add an additional VBS filter file

        @param filter: VBS filter file
        @type filter: C{str}
        @return: None
        """

    def add_textfilter(filter):
        """
        add_textfilter

        Add an additional Text filter file

        @param filter: Text filter file
        @type filter: C{str}
        @return: None
        """

    def add_samplefilter(filter):
        """
        add_samplefilter

        Add an additional sample filter file

        @param filter: Sample filter file
        @type filter: C{str}
        @return: None
        """

    def log_event():
        """
        log_event

        Log the URL analysis results

        @return None
        """

    def run_local(url):
        """
        run_local

        This method should be invoked by 'analyze' method for local file analysis

        @param url: URL to analyze
        @type url: C{str}
        """

    def run_remote(url):
        """
        run_remote

        This method should be invoked by 'run' method for URL analysis

        @param url: URL to analyze
        @type url: C{str}
        """

    def analyze():
        """
        analyze

        This method is called when the ThugAPI subclass is called and MUST be
        implemented. This method can reference just the 'args' class attribute.
        Returning something from this method is up to you if needed.
        """



# Vantage Linguistics AnserWorks ActiveX Controls
# CVE-2007-6387

import logging

log = logging.getLogger("Thug")


def GetHistory(self, arg):
    if len(arg) > 215:
        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6387", None)
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "AnswerWorks ActiveX",
                                          "Overflow in GetHistory",
                                          cve = 'CVE-2007-6387')


def GetSeedQuery(self, arg):
    if len(arg) > 215:
        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6387", None)
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "AnswerWorks ActiveX",
                                          "Overflow in GetSeedQuery",
                                          cve = 'CVE-2007-6387')


def SetSeedQuery(self, arg):
    if len(arg) > 215:
        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6387", None)
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "AnswerWorks ActiveX",
                                          "SetSeedQuery",
                                          cve = 'CVE-2007-6387')




import logging

log = logging.getLogger("Thug")


def ShellExecute(self, *args):
    cmdLine = ''

    for arg in args:
        if not arg or len(arg) == 0:
            continue

        cmdLine += str(arg)

    # Attempt to extract some URLs from the command line

    urls = set()

    if 'http' in cmdLine:
        for sep in ("'", '"'):
            offset = cmdLine.find("{}http".format(sep))
            if offset < 0:
                continue

            url = cmdLine[offset + 1:].split("'")
            urls.add(url[0])

    log.ThugLogging.add_behavior_warn('[Shell.Application ActiveX] ShellExecute command: {}'.format(cmdLine))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Shell.Application ActiveX",
                                      "ShellExecute command",
                                      data = {
                                                "command" : cmdLine
                                             },
                                      forward = False)

    for url in urls:
        log.ThugLogging.add_behavior_warn('[Shell.Application ActiveX] URL detected: {}'.format(url))

        try:
            self._window._navigator.fetch(url, redirect_type = "ShellExecute")
        except: # pylint:disable=bare-except
            pass



#!/usr/bin/env python
#
# HPFeeds.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import os
import struct
import socket
import hashlib
import logging
import json

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser

log = logging.getLogger("Thug")


class FeedUnpack(object):
    def __init__(self):
        self.buf = bytearray()

    def __iter__(self):
        return self

    def next(self):
        return self.unpack()

    def feed(self, data):
        self.buf.extend(data)

    def unpack(self):
        if len(self.buf) < 5:
            raise StopIteration('No message')

        ml, opcode = struct.unpack('!iB', buffer(self.buf, 0, 5))
        if len(self.buf) < ml:
            raise StopIteration('No message')

        data = bytearray(buffer(self.buf, 5, ml - 5))
        del self.buf[:ml]
        return opcode, data


class HPFeeds(object):
    formats = ('maec11', )

    OP_ERROR        = 0
    OP_INFO         = 1
    OP_AUTH         = 2
    OP_PUBLISH      = 3
    OP_SUBSCRIBE    = 4

    def __init__(self, thug_version):
        self.unpacker     = FeedUnpack()
        self.thug_version = thug_version
        self.opts         = dict()
        self.url          = ""
        self.enabled      = True

        self.__init_config()

    def __init_config(self):
        config = ConfigParser.ConfigParser()

        conf_file = os.path.join(log.configuration_path, 'logging.conf')

        if not os.path.exists(conf_file):
            conf_file = os.path.join(log.configuration_path, 'logging.conf.default')

        if not os.path.exists(conf_file):
            self.enabled = False
            return

        config.read(conf_file)

        for option in config.options('hpfeeds'):
            self.opts[option] = str(config.get('hpfeeds', option))

    def set_url(self, url):
        self.url = url

    def msg_hdr(self, op, data):
        return struct.pack('!iB', 5 + len(data), op) + data

    def msg_publish(self, chan, data):
        # if isinstance(data, str):
        #    data = data.encode('latin1')

        return self.msg_hdr(self.OP_PUBLISH,
                            struct.pack('!B', len(self.opts['ident'])) +
                            self.opts['ident'] +
                            struct.pack('!B', len(chan)) +
                            chan +
                            data)

    def msg_auth(self, rand):
        _hash = hashlib.sha1(rand + self.opts['secret']).digest()
        return self.msg_hdr(self.OP_AUTH,
                            struct.pack('!B', len(self.opts['ident'])) +
                            self.opts['ident'] +
                            _hash)

    def msg_send(self, msg):
        self.sockfd.send(msg)

    def get_data(self, host, port):
        self.sockfd.settimeout(3)

        try:
            self.sockfd.connect((host, port))
        except:  # pylint:disable=bare-except
            log.warning('[HPFeeds] Unable to connect to broker')
            return None

        try:
            d = self.sockfd.recv(1024)
        except socket.timeout:
            log.warning('[HPFeeds] Timeout on banner')
            return None

        self.sockfd.settimeout(None)
        return d

    def publish_data(self, d, chan, pubdata):
        published = False

        while d and not published:
            self.unpacker.feed(d)

            for opcode, data in self.unpacker:
                if opcode == self.OP_INFO:
                    rest = buffer(data, 0)
                    name, rest = rest[1:1 + ord(rest[0])], buffer(rest, 1 + ord(rest[0]))  # pylint:disable=unused-variable
                    rand = str(rest)

                    self.msg_send(self.msg_auth(rand))
                    self.msg_send(self.msg_publish(chan, pubdata))
                    published = True
                    self.sockfd.settimeout(0.1)
                if opcode == self.OP_ERROR:
                    log.warning('[HPFeeds] Error message from server: %s', data)

            try:
                d = self.sockfd.recv(1024)
            except socket.timeout:
                break

    def __log_event(self, pubdata):
        self.sockfd = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        data = self.get_data(self.opts['host'], int(self.opts['port']))
        if data is None:
            return

        self.publish_data(data, 'thug.events', pubdata)
        self.sockfd.close()

    def log_event(self, basedir):
        if not self.enabled:
            return

        if log.ThugOpts.local:
            return

        m = None

        for module in self.formats:
            if module in log.ThugLogging.modules:
                p = log.ThugLogging.modules[module]
                m = getattr(p, 'get_%s_data' % (module, ), None)
                if m:
                    break

        if m is None:
            return

        data = m(basedir)
        self.__log_event(data)

    def log_file(self, pubdata, url = None, params = None):
        if not self.enabled:
            return

        if log.ThugOpts.local:
            return

        self.sockfd = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        data = self.get_data(self.opts['host'], int(self.opts['port']))
        if data is None:
            return

        self.publish_data(data, 'thug.files', json.dumps(pubdata))
        self.sockfd.close()

    def log_warning(self, pubdata):
        if not self.enabled:
            return

        if log.ThugOpts.local:
            return

        self.sockfd = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        data = self.get_data(self.opts['host'], int(self.opts['port']))
        if data is None:
            return

        self.publish_data(data, 'thug.warnings', json.dumps({'url': self.url, 'warning': pubdata}))
        self.sockfd.close()


if __name__ == '__main__':
    hpfeeds = HPFeeds('0.8.19')
    hpfeeds.log_event('Test foobar!')



# Nessus Vunlnerability Scanner ScanCtrl ActiveX Control
# CVE-2007-4061, CVE-2007-4062, CVE-2007-4031

import logging

log = logging.getLogger("Thug")


def deleteReport(self, arg):
    log.ThugLogging.add_behavior_warn('[Nessus Vunlnerability Scanner ScanCtrl ActiveX] deleteReport(%s)' % (arg, ),
                                      'CVE-2007-4031')
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Nessus Vunlnerability Scanner ScanCtrl ActiveX",
                                      "deleteReport",
                                      cve = "CVE-2007-4031",
                                      data = {
                                                "arg": arg
                                             },
                                      forward = False)

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4031", None)

def deleteNessusRC(self, arg):
    log.ThugLogging.add_behavior_warn('[Nessus Vunlnerability Scanner ScanCtrl ActiveX] deleteNessusRC(%s)' % (arg, ),
                                      'CVE-2007-4062')
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Nessus Vunlnerability Scanner ScanCtrl ActiveX",
                                      "deleteNEssusRC",
                                      cve = "CVE-2007-4062",
                                      data = {
                                                "arg": arg
                                             },
                                      forward = False)

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4062", None)


def saveNessusRC(self, arg):
    log.ThugLogging.add_behavior_warn('[Nessus Vunlnerability Scanner ScanCtrl ActiveX] saveNessusRC(%s)' % (arg, ),
                                      'CVE-2007-4061')
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Nessus Vunlnerability Scanner ScanCtrl ActiveX",
                                      "saveNessusRC",
                                      cve = "CVE-2007-4061",
                                      data = {
                                                "arg": arg
                                             },
                                      forward = False)

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4061", None)


def addsetConfig(self, arg, arg1, arg2):
    log.ThugLogging.add_behavior_warn('[Nessus Vunlnerability Scanner ScanCtrl ActiveX] addsetConfig(%s, %s, %s)' % (arg, arg1, arg2, ),
                                      'CVE-2007-4061')
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Nessus Vunlnerability Scanner ScanCtrl ActiveX",
                                      "addsetConfig",
                                      cve = "CVE-2007-4061",
                                      data = {
                                                "arg" : arg,
                                                "arg1": arg1,
                                                "arg2": arg2
                                             },
                                      forward = False)

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4061", None)



#!/usr/bin/env python

import logging
import bs4 as BeautifulSoup
from thug.DOM.W3C.DOMException import DOMException
from .HTMLElement import HTMLElement
from .HTMLCollection import HTMLCollection
from .HTMLTableRowElement import HTMLTableRowElement
from .attr_property import attr_property

log = logging.getLogger("Thug")


class HTMLTableSectionElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)
        self._rows = HTMLCollection(doc, list())

    align           = attr_property("align")
    ch              = attr_property("char")
    chOff           = attr_property("charoff")
    vAlign          = attr_property("valign")

    @property
    def rows(self):
        return self._rows

    # Modified in DOM Level 2
    def insertRow(self, index = None):
        # `index' specifies the position of the row to insert (starts at 0). The value of
        # -1 can also be used; which result in that the new row will be inserted at the
        # last position. This parameter is required in Firefox and Opera, but optional in
        # Internet Explorer, Chrome and Safari. If this parameter is omitted, insertRow()
        # inserts a new row at the last position in IE and at the first position in Chrome
        # and Safari.
        if index is None:
            if log.ThugOpts.Personality.isIE():
                index = -1
            if log.ThugOpts.Personality.isChrome() or log.ThugOpts.Personality.isSafari():
                index = 0

        row = HTMLTableRowElement(self.doc, BeautifulSoup.Tag(self.doc, name = 'tr'))

        if index in (-1, len(self._rows), ):
            self.rows.nodes.append(row)
        else:
            self.rows.nodes.insert(index, row)

        return row

    # Modified in DOM Level 2
    def deleteRow(self, index):
        if index < -1 or index >= len(self.rows.nodes):
            raise DOMException(DOMException.INDEX_SIZE_ERR)

        del self.rows.nodes[index]



# RealMedia RealPlayer Ierpplug.DLL ActiveX Control
# CVE-2007-5601

import logging

log = logging.getLogger("Thug")


def DoAutoUpdateRequest(self, arg0, arg1, arg2):
    if len(arg0) > 1000 or len(arg1) > 1000:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "RealMedia RealPlayer Ierpplug.DLL ActiveX",
                                          "Overflow in DoAutoUpdateRequest",
                                          cve = "CVE-2007-5601")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-5601", None)


def PlayerProperty(self, arg):
    if arg == 'PRODUCTVERSION':
        return '6.0.14.552'

    if len(arg) > 1000:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "RealMedia RealPlayer Ierpplug.DLL ActiveX",
                                          "Overflow in PlayerProperty",
                                          cve = "CVE-2007-5601")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-5601", None)


def Import(self, arg):
    if len(arg) > 0x8000:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "RealMedia RealPlayer Ierpplug.DLL ActiveX",
                                          "Overflow in Import",
                                          cve = "CVE-2007-5601")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-5601", None)


def SetConsole(self, val):
    self.__dict__['Console'] = val

    if len(val) >= 32:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "RealMedia RealPlayer rmoc3260.DLL ActiveX",
                                          "Overflow in Console property",
                                          cve = "CVE-2007-5601")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-5601", None)



# StreamAudio ChainCast VMR Client Proxy ActiveX Control 3.x
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def InternalTuneIn(self, arg0, arg1, arg2, arg3, arg4):
    if len(arg0) > 248:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "StreamAudio ChainCast VMR Client Proxy ActiveX",
                                          "Buffer overflow in arg0")



# MPS.StormPlayer.1  'advanceOpen'
# CVE

import logging

log = logging.getLogger("Thug")


def advancedOpen(self, arg0, arg1):
    if len(arg0) > 259:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "MPS.StormPlayer.1 ActiveX",
                                          "advanceOpen Method Overflow")


def isDVDPath(self, arg0):
    if len(arg0) > 246:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "MPS.StormPlayer.1 ActiveX",
                                          "isDVDPath Method Overflow")


def rawParse(self, arg0):
    if len(arg0) > 259:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "MPS.StormPlayer.1 ActiveX",
                                          "rawParse Method Overflow")


def OnBeforeVideoDownload(self, arg0):
    if len(arg0) > 4124:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "MPS.StormPlayer.1 ActiveX",
                                          "OnBeforeVideoDownload Method Overflow")


def SetURL(self, val):
    self.__dict__['URL'] = val

    if len(val) > 259:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "MPS.StormPlayer.1 ActiveX",
                                          "URL Console Overflow")


def SetbackImage(self, val):
    self.__dict__['backImage'] = val

    if len(val) > 292:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "MPS.StormPlayer.1 ActiveX",
                                          "backImage Console Overflow")


def SettitleImage(self, val):
    self.__dict__['titleImage'] = val

    if len(val) > 296:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "MPS.StormPlayer.1 ActiveX",
                                          "titleImage Console Overflow")




import logging

log = logging.getLogger("Thug")


def launch(self, arg):
    log.ThugLogging.add_behavior_warn("[Java Deployment Toolkit ActiveX] Launching: %s" % (arg, ))

    tokens = arg.split(' ')
    if tokens[0].lower() != 'http:':
        return

    for token in tokens[1:]:
        if not token.lower().startswith('http'):
            continue

        log.ThugLogging.add_behavior_warn("[Java Deployment Toolkit ActiveX] Fetching from URL %s" % (token, ))
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Java Deployment Toolkit ActiveX",
                                          "Fetching from URL",
                                          data = {
                                                    "url": token
                                                 },
                                          forward = False)

        try:
            self._window._navigator.fetch(token, redirect_type = "Java Deployment Toolkit Exploit")
        except:  # pylint:disable=bare-except
            log.ThugLogging.add_behavior_warn("[Java Deployment Toolkit ActiveX] Fetch Failed")


def launchApp(self, pJNLP, pEmbedded = None, pVmArgs = None):
    cve_2013_2416 = False
    if len(pJNLP) > 256:
        cve_2013_2416 = True
        log.DFT.check_shellcode(pJNLP)

    if pEmbedded and len(pEmbedded):
        cve_2013_2416 = True
        log.DFT.check_shellcode(pEmbedded)

    if cve_2013_2416:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Java Deployment Toolkit ActiveX",
                                          "Java ActiveX component memory corruption (CVE-2013-2416)",
                                          cve = "CVE-2013-2416",
                                          forward = True)

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2013-2416", None)



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property

import logging
log = logging.getLogger("Thug")


class HTMLMetaElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    content         = attr_property("content")
    httpEquiv       = attr_property("http-equiv")
    name            = attr_property("name", default = "")
    scheme          = attr_property("scheme")
    _charset        = attr_property("charset", default = "")

    def __getattr__(self, name):
        if name in ('charset', ) and log.ThugOpts.Personality.isIE():
            return self._charset

        raise AttributeError



# Symantec BackupExec
# CVE-2007-6016,CVE-2007-6017

import logging

log = logging.getLogger("Thug")


def Set_DOWText0(self, val):
    self.__dict__['_DOWText0'] = val

    if len(val) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Symantec BackupExec ActiveX",
                                          "Overflow in property _DOWText0",
                                          cve = 'CVE-2007-6016')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6016", None)


def Set_DOWText6(self, val):
    self.__dict__['_DOWText6'] = val

    if len(val) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Symantec BackupExec ActiveX",
                                          "Overflow in property _DOWText6",
                                          cve = 'CVE-2007-6016')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6016", None)


def Set_MonthText0(self, val):
    self.__dict__['_MonthText0'] = val

    if len(val) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Symantec BackupExec ActiveX",
                                          "Overflow in property _MonthText6",
                                          cve = 'CVE-2007-6016')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6016", None)


def Set_MonthText11(self, val):
    self.__dict__['_MonthText11'] = val

    if len(val) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Symantec BackupExec ActiveX",
                                          "Overflow in property _MonthText11",
                                          cve = 'CVE-2007-6016')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6016", None)


def Save(self, a, b):
    return



# MySpace Uploader Control 1.x
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def SetAction(self, val):
    self.__dict__['Action'] = val

    if len(val) > 512:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Myspace UPloader ActiveX",
                                          "Overflow in Action property")
